local LMP = LibStub('LibMediaProvider-1.0')

-- Setting classId constants (not found in global constants...)
local CLASS_NIGHTBLADE 	= 3
local CLASS_SORCERER 	= 2
local CLASS_TEMPLAR		= 6
local CLASS_DRAGONKNIGHT= 1

-- alliance icons texture tables
local allianceIcons = {
	[ALLIANCE_ALDMERI_DOMINION]		= '/E/art/guild/guildbanner_icon_aldmeri.dds',--/esoui/art/contacts/social_allianceicon_aldmeri.dds',
	[ALLIANCE_DAGGERFALL_COVENANT] 	= '/esoui/art/guild/guildbanner_icon_daggerfall.dds',--/esoui/art/contacts/social_allianceicon_daggerfall.dds',
	[ALLIANCE_EBONHEART_PACT]		= '/esoui/art/guild/guildbanner_icon_ebonheart.dds'--/esoui/art/contacts/social_allianceicon_ebonheart.dds'
	}

local allianceIconsColor = {
	[ALLIANCE_ALDMERI_DOMINION]		= {0,0,200/255,1},
	[ALLIANCE_DAGGERFALL_COVENANT] 	= {200/255,200/255,0,1},
	[ALLIANCE_EBONHEART_PACT]		= {200/255,0,0,1}
	}
	
-- class icons texture tables
local classIcons = {
	[CLASS_DRAGONKNIGHT] 	= '/esoui/art/contacts/social_classicon_dragonknight.dds',
	[CLASS_NIGHTBLADE]   	= '/esoui/art/contacts/social_classicon_nightblade.dds',
	[CLASS_SORCERER]		= '/esoui/art/contacts/social_classicon_sorcerer.dds',
	[CLASS_TEMPLAR]			= '/esoui/art/contacts/social_classicon_templar.dds'
	}
	
local classIconsColor = {	
	[CLASS_DRAGONKNIGHT] 	= {200/255,100/255,0,0},
	[CLASS_NIGHTBLADE]   	= {255/255,0,0,1},
	[CLASS_SORCERER]		= {1,0,255/255,1},
	[CLASS_TEMPLAR]			= {255/255,255/255,100/255,1}
}

-- bars/attribute mapping,
CurvyHud.barPowertypeMapping = {
	['healthBar'] 	= POWERTYPE_HEALTH,
	['staminaBar'] 	= POWERTYPE_STAMINA,
	['magickaBar'] 	= POWERTYPE_MAGICKA,
	['mountBar'] 	= POWERTYPE_MOUNT_STAMINA,
	['werewolfBar'] = POWERTYPE_WEREWOLF,
}

-- attribute/bars mapping,
CurvyHud.powerTypeBarMapping = {
	[POWERTYPE_HEALTH] 			= 'healthBar',
	[POWERTYPE_STAMINA] 		= 'staminaBar',
	[POWERTYPE_MAGICKA] 		= 'magickaBar',
	[POWERTYPE_MOUNT_STAMINA] 	= 'mountBar',
	[POWERTYPE_WEREWOLF]		= 'werewolfBar',
}

-- stat type/bars mapping, PLAYER ONLY
CurvyHud.statTypeBarMapping = {
	[STAT_HEALTH_REGEN_COMBAT] 			= 'healthBar',
	[STAT_STAMINA_REGEN_COMBAT] 		= 'staminaBar',
	[STAT_MAGICKA_REGEN_COMBAT] 		= 'magickaBar'
}

local barList = { 
	[1] = 'healthBar',
	[2] = 'targetBar',
	[3] = 'magickaBar',
	[4] = 'staminaBar',
	[5] = 'mountBar',
	[6] = 'siegeBar',
	[7] = 'werewolfBar',}--'ultiBar','bossBar'}

---------------------------
-- Local utils functions --
---------------------------

-- round a number to closest integer
local function round(num) 
	if num >= 0 then return math.floor(num+.5) 
	else return math.ceil(num-.5) end
end

-- deepcopy a table
local function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end


-- Format attribute value with the given text format
local function FormatTextValues(textFormat,currentValue,maxValue,percentValue)
	local text = string.gsub(textFormat,'{val}',tostring(currentValue))
	text = string.gsub(text,'{per}',tostring(percentValue))
	text = string.gsub(text,'{max}',tostring(maxValue))
	return text
end


-------------------------------
-- Initializing and building --
-------------------------------

-- Initialize all controls
function CurvyHud:InitializeControls()

	local topControl = self.topControl or  WINDOW_MANAGER:CreateTopLevelWindow("CURVY_TopControl") 
	self.topControl = topControl
	topControl:SetDimensions(GuiRoot:GetWidth(),GuiRoot:GetHeight())
	topControl:SetAnchor(TOPLEFT)
	
	self:BuildBarContainers()
	self:BuildTargetNameplate()
	self:BuildInteractionPrompt()
	
	self:ToggleDefaultPlayerFrame()
	self:ToggleDefaultTargetFrame()
	
	
	-- TEST CONTROL --
	-- local c =   WINDOW_MANAGER:CreateTopLevelWindow(nil) 
	-- local pouet = WINDOW_MANAGER:CreateControl('pouet',c,CT_TEXTURE)
	-- pouet:SetAnchor(CENTER,GuiRoot,CENTER)
	-- pouet:SetTexture(self.name..'/pouet.dds')
	-- pouet:SetDimensions(200,80)
	-- pouet:SetColor(1,0,0,1)
	-- pouet:SetTextureRotation(math.pi/2,0.5,0.5)

end

-- Build the bar containers
function CurvyHud:BuildBarContainers()
	-- useful locals
	local cfg = self.config
	local textureCfg = self.textures[cfg.textureConfig]
	
	-- Creating and positioning the left and right containers 
	local containerLeft = self.barContainerLeft or WINDOW_MANAGER:CreateControl("CURVY_BarContainer_Left",self.topControl,CT_CONTROL)
	self.barContainerLeft = containerLeft
	cfg.barContainerLeft.anchor[2] = GuiRoot
	containerLeft:ClearAnchors()
	containerLeft:SetAnchor(unpack(cfg.barContainerLeft.anchor))
	containerLeft.count = 0
	containerLeft.bars = {}
	-- containerLeft:SetHidden(false)
	
	local containerRight = self.barContainerRight or WINDOW_MANAGER:CreateControl("CURVY_BarContainer_Right",self.topControl,CT_CONTROL) 
	self.barContainerRight = containerRight
	cfg.barContainerRight.anchor[2] = GuiRoot
	containerRight:ClearAnchors()
	containerRight:SetAnchor(unpack(cfg.barContainerRight.anchor))
	containerRight.count = 0
	containerRight.bars = {}
	-- containerRight:SetHidden(false)
	
	-- Creating the attribute bars
	for i, barName in ipairs(barList) do
		self:BuildBar(barName)
		self:BuildTextValues(barName)

		if(cfg[barName].position == 'left') then
			containerLeft.count 	= containerLeft.count 	+ 1
		else
			containerRight.count 	= containerRight.count 	+ 1
		end
	end
	
	-- Finally, setting the dimensions of the bar containers
	local barThickness = (textureCfg.barThickness/textureCfg.barWidth) * cfg.barWidth
	
	containerLeft :SetDimensions(cfg.barWidth + (cfg.barSpacing + barThickness)*2 ,cfg.barHeight)
	containerRight:SetDimensions(cfg.barWidth + (cfg.barSpacing + barThickness)*2 ,cfg.barHeight)
	
	-- adding containers to movable frames
	self.movableFrames.barContainerLeft = containerLeft
	self.movableFrames.barContainerLeft.anchorConfig = cfg.barContainerLeft.anchor
	self.movableFrames.barContainerRight = containerRight
	self.movableFrames.barContainerRight.anchorConfig = cfg.barContainerRight.anchor
end

-- Build the given bar
function CurvyHud:BuildBar(barName)
	-- useful locals
	local cfg 			= self.config
	local textureCfg 	= self.textures[cfg.textureConfig]
	local barCfg		= self.config[barName]
	local visualsCfg 	= (barName == 'targetBar' and cfg.targetVisuals or cfg.visuals)
	
	--
	-- Pre-calculating positioning values
	--
	
	-- Setting the container, texture file and anchor depending on the position (left or right)
	local textureFile  		= textureCfg.fileRight
	local container			= self.barContainerRight
	local containerConfig 	= cfg.barContainerRight
	local anchor 			= LEFT
	local x 				= 1;
	
	if (barCfg.position == 'left') then
		x					= -1
		textureFile 		= textureCfg.fileLeft
		container 			= self.barContainerLeft
		containerConfig 	= cfg.barContainerLeft
		anchor 				= RIGHT
	end
	
	-- setting 'step size' for stairway style 
	local stepSize = (containerConfig.stepOn and textureCfg.stepSize or 0)
	
	-- setting inGame/texture aspect ratio 
	local ratioH = cfg.barHeight/textureCfg.barHeight -- vertical ratio, H stands for Height
	local ratioW = cfg.barWidth/textureCfg.barWidth	  -- horizontal ratio, W stands for Width	
	
	-- Calculating horizontal offset, which effectively sets the position in the container
	local barThickness = round(textureCfg.barThickness * ratioW)
	local xOffset = (barThickness+cfg.barSpacing)*barCfg.index*x
	
	--
	-- Creating the main bar frame, containing all others
	--	
	local bar 	= self.bars[barName] or self[barName] or WINDOW_MANAGER:CreateControl("CURVY_"..barName,container,CT_CONTROL)
	self.bars[barName] = bar
	container.bars[barName] = bar
	bar:ClearAnchors()
	bar:SetAnchor(anchor,container,anchor,xOffset,0)
	bar:SetDimensions(cfg.barWidth,cfg.barHeight)
	
	-- saving config in the bar for easy access later
	bar.config=barCfg
	
	--
	-- Creating Background
	--	
	local backgroundStepSize 		= barCfg.index*stepSize
	local backgroundStepSizeT		= 0
	local backgroundHeight 			=  cfg.barHeight - (backgroundStepSize+backgroundStepSizeT)*ratioH
	local backgroundYOffset 		= -backgroundStepSize*ratioH
	local backgroundTextureCoords 	= deepcopy(textureCfg.coords['background'])
	backgroundTextureCoords[3] 		= backgroundTextureCoords[3] + backgroundStepSizeT/textureCfg.textureHeight
	backgroundTextureCoords[4] 		= backgroundTextureCoords[4] - backgroundStepSize /textureCfg.textureHeight
	
	local background = 	bar.background or WINDOW_MANAGER:CreateControl(nil,bar,CT_TEXTURE)
	bar.background = background
	background:ClearAnchors()
	background:SetTexture(textureFile)
	background:SetTextureCoords(unpack(backgroundTextureCoords))
	background:SetDimensions(cfg.barWidth,backgroundHeight)
	background:SetAnchor(BOTTOMLEFT,bar,BOTTOMLEFT,0,backgroundYOffset)
	background:SetColor(unpack(barCfg.backgroundColor))
	
	
	--
	-- Creating Left and right border 
	--	
	local borderStepSize 		= backgroundStepSize 	+ textureCfg.borderMargin + textureCfg.borderSize
	local borderStepSizeT 		= backgroundStepSizeT 	+ textureCfg.borderMargin + textureCfg.borderSize
	local borderHeight 			= cfg.barHeight - (borderStepSize+borderStepSizeT)*ratioH
	local borderYOffset 		= - borderStepSize*ratioH
	local borderTextureCoords 	= deepcopy(textureCfg.coords['border'])
	borderTextureCoords[3] 		= borderTextureCoords[3] + borderStepSizeT/textureCfg.barHeight
	borderTextureCoords[4] 		= borderTextureCoords[4] - borderStepSize /textureCfg.textureHeight

	local border = bar.border  or WINDOW_MANAGER:CreateControl(nil,bar,CT_TEXTURE)	
	bar.border = border
	border:ClearAnchors()
	border:SetColor(unpack(barCfg.borderColorMax))
	border:SetTexture(textureFile)
	border:SetTextureCoords(unpack(borderTextureCoords))
	border:SetDimensions(cfg.barWidth,borderHeight)
	border:SetAnchor(BOTTOMLEFT,bar,BOTTOMLEFT,0,borderYOffset)
	
	--
	-- Creating Filler
	--	
	local fillerStepSize 		= borderStepSize + textureCfg.borderMargin
	local fillerStepSizeT 		= borderStepSizeT+ textureCfg.borderMargin
	local fillerHeight			=  cfg.barHeight - (fillerStepSize+fillerStepSizeT)*ratioH
	local fillerStepYOffset 	= -fillerStepSize*ratioH
	local fillerTCoords	= deepcopy(textureCfg.coords['filler'])
	fillerTCoords[3]		= fillerTCoords[3] + fillerStepSizeT/textureCfg.textureHeight
	fillerTCoords[4]		= fillerTCoords[4] - fillerStepSize /textureCfg.textureHeight
	
	local filler = bar.filler or WINDOW_MANAGER:CreateControl(nil,bar,CT_TEXTURE)
	bar.filler = filler
	bar.filler:ClearAnchors()	
	filler:SetColor(unpack(barCfg.colorMax))
	filler:SetTexture(textureFile)
	filler:SetTextureCoords(unpack(fillerTCoords))
	filler:SetDimensions(cfg.barWidth,fillerHeight)
	filler:SetAnchor(BOTTOMLEFT,bar,BOTTOMLEFT,0,fillerStepYOffset)
	-- saving filler infos for update of the bar later
	filler.baseHeight = fillerHeight
	filler.baseTextureCoords = deepcopy(fillerTCoords)
	
	--
	-- Creating shield bar
	--	
	-- obviously, only in healthBar and targetBar
	if(barName == "healthBar" or barName == 'targetBar') then
		local shield = bar.shield or WINDOW_MANAGER:CreateControl(nil,bar,CT_TEXTURE)
		bar.shield = shield
		bar.shield:ClearAnchors()
		
		shield:SetColor(unpack(visualsCfg.shield.color))
		shield:SetTexture(textureFile)
		shield:SetTextureCoords(unpack(fillerTCoords))
		shield:SetDimensions(cfg.barWidth,0)
		shield:SetAnchor(TOPLEFT,bar,TOPLEFT,0,fillerStepSizeT*ratioH)
		shield:SetHidden(false)
		-- saving shield bar infos for update later
		shield.baseAnchor = {TOPLEFT,bar,TOPLEFT,0,fillerStepSizeT*ratioH}
	end
	
	--
	-- Creating Regen
	-- 	
	local regen = bar.regen or WINDOW_MANAGER:CreateControl(nil,bar,CT_TEXTURE)
	bar.regen = regen
	bar.regen:ClearAnchors()

	local regenTCoords =  deepcopy(textureCfg.coords['regen'])
	regen:SetInheritAlpha(false)
	regen:SetTexture(textureFile)
	regen:SetTextureCoords(
		regenTCoords[1],
		regenTCoords[2],
		fillerTCoords[4],
		fillerTCoords[4])
	regen:SetDimensions(cfg.barWidth,0)
	regen:SetHidden(true)
	regen.baseTextureCoords = regenTCoords
	
	--
	-- Creating Degen
	-- 	
	local degen = bar.degen or WINDOW_MANAGER:CreateControl(nil,bar,CT_TEXTURE)
	bar.degen = degen
	bar.degen:ClearAnchors()

	local degenTCoords =  deepcopy(textureCfg.coords['degen'])
	degen:SetInheritAlpha(false)
	degen:SetTexture(textureFile)
	degen:SetTextureCoords(
		degenTCoords[1],
		degenTCoords[2],
		fillerTCoords[4],
		fillerTCoords[4])
	degen:SetDimensions(cfg.barWidth,0)
	degen:SetHidden(true)
	degen.baseTextureCoords = degenTCoords
	
	--
	-- Creating Bottom border
	--	
	local bottomBorderStepSize 	= backgroundStepSize + textureCfg.borderMargin
	local bottomBorderStepSizeT	= textureCfg.barHeight - bottomBorderStepSize - textureCfg.borderSize
	local bottomBorderHeight 	= textureCfg.borderSize*ratioH
	local bottomBorderYOffset 	= -bottomBorderStepSize * ratioH
	local bottomBorderCoords 	= deepcopy(textureCfg.coords['full'])
	bottomBorderCoords[3] 		= bottomBorderCoords[3] + bottomBorderStepSizeT/textureCfg.textureHeight
	bottomBorderCoords[4] 		= bottomBorderCoords[4] - bottomBorderStepSize /textureCfg.textureHeight
	
	local bottomBorder = bar.bottomBorder or WINDOW_MANAGER:CreateControl(nil,bar,CT_TEXTURE)
	bar.bottomBorder = bottomBorder
	bottomBorder:ClearAnchors()
	bottomBorder:SetAnchor(BOTTOMLEFT,bar,BOTTOMLEFT,0,bottomBorderYOffset)
	
	bottomBorder:SetColor(unpack(barCfg.borderColorMax))
	bottomBorder:SetTexture(textureFile)
	bottomBorder:SetTextureCoords(unpack(bottomBorderCoords))
	bottomBorder:SetDimensions(cfg.barWidth,bottomBorderHeight)
	
	--
	-- Creating Top border
	--
	local topBorderStepSizeT 	= backgroundStepSizeT + textureCfg.borderMargin
	local topBorderStepSize 	= textureCfg.barHeight - topBorderStepSizeT - textureCfg.borderSize
	local topBorderHeight 		= bottomBorderHeight
	local topBorderYOffset 		= -topBorderStepSize * ratioH
	local topBorderCoords	 	= deepcopy(textureCfg.coords['full'])
	topBorderCoords[3] 			= topBorderCoords[3] + topBorderStepSizeT/textureCfg.textureHeight
	topBorderCoords[4]			= topBorderCoords[4] - topBorderStepSize /textureCfg.textureHeight
	
	local topBorder = bar.topBorder or WINDOW_MANAGER:CreateControl(nil,bar,CT_TEXTURE)
	bar.topBorder = topBorder
	topBorder:ClearAnchors()
	topBorder:SetAnchor(BOTTOMLEFT,bar,BOTTOMLEFT,0,topBorderYOffset)
	
	topBorder:SetColor(unpack(barCfg.borderColorMax))
	topBorder:SetTexture(textureFile)
	topBorder:SetTextureCoords(unpack(topBorderCoords))
	topBorder:SetDimensions(cfg.barWidth,topBorderHeight)
	
	--
	-- Coloring
	--
	-- Calculating border & filler colors difference between colorLow and colorMax
	-- (used later when updating the bar)
	border.rGap = barCfg.borderColorMax[1]-barCfg.borderColorLow[1]
	border.gGap = barCfg.borderColorMax[2]-barCfg.borderColorLow[2]
	border.bGap = barCfg.borderColorMax[3]-barCfg.borderColorLow[3]
	border.aGap = barCfg.borderColorMax[4]-barCfg.borderColorLow[4]
	
	filler.rGap = barCfg.colorMax[1]-barCfg.colorLow[1]
	filler.gGap = barCfg.colorMax[2]-barCfg.colorLow[2]
	filler.bGap = barCfg.colorMax[3]-barCfg.colorLow[3]
	filler.aGap = barCfg.colorMax[4]-barCfg.colorLow[4]

	--
	-- Animating
	--
	if((barName == 'healthBar' and self.config.visuals.shield.show) or (barName == 'targetBar' and self.config.targetVisuals.shield.show)) then
		self.CreateShieldAnim(barName,bar)
	else
		self.CreateBaseAnim(barName,bar)
	end
	self.CreateRegenAnim(barName,bar,fillerStepSize)
	self.CreateDegenAnim(barName,bar,fillerStepSizeT)
end

-- Build the target nameplate
function CurvyHud:BuildTargetNameplate()	
	-- useful locals
	local cfg = self.config
	local nameplateCfg = self.config.targetNameplate
	
	
	-- font nameplateCfguration
	local font = string.format('%s|%d|%s', LMP:Fetch('font', nameplateCfg.fontName), nameplateCfg.fontSize, 'soft-shadow-thick')
	local fontCaption = string.format('%s|%d|%s', LMP:Fetch('font', nameplateCfg.fontName), nameplateCfg.fontSize-4, 'soft-shadow-thick')
	local iconSize = nameplateCfg.fontSize * 1.5
	local fontDifficulty = string.format('%s|%d|%s', LMP:Fetch('font', nameplateCfg.fontName), nameplateCfg.fontSize*2, 'soft-shadow-thick')
	
	--anchors and offsets nameplateCfguration depending on alignment
	local top = BOTTOM
	local topRelative = TOP
	local bottom = TOP
	local bottomRelative = BOTTOM
	local x = 0
	-- if(nameplateCfg.alignment == 'left') then
		-- top = BOTTOMLEFT
		-- topRelative = TOPLEFT
		-- bottom = TOPLEFT
		-- bottomRelative = BOTTOMLEFT
		-- x=1
	-- elseif(nameplateCfg.alignment == 'right') then
		-- top = BOTTOMRIGHT
		-- topRelative = TOPRIGHT
		-- bottom = TOPRIGHT
		-- bottomRelative = BOTTOMRIGHT
		-- x=-1
	-- end
	
	-- Creating and positioning the target nameplate container
	local container = self.targetNameplate or WINDOW_MANAGER:CreateControl('CURVY_Target_Nameplate',self.topControl,CT_CONTROL)
	self.targetNameplate = container
	nameplateCfg.anchor[2] = GuiRoot
	container:ClearAnchors()
	container:SetAnchor(unpack(nameplateCfg.anchor))
	
	
	local name = container.name or WINDOW_MANAGER:CreateControl(nil,container,CT_LABEL)
	name:SetFont(font)
	name:ClearAnchors()
	name:SetAnchor(CENTER,container,CENTER,0,0)
	name:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
	name:SetVerticalAlignment(TEXT_ALIGN_TOP)
	name:SetHeight(nameplateCfg.fontSize +10)
	container.name = name

	local level = container.level or WINDOW_MANAGER:CreateControl(nil,container,CT_LABEL)
	level:SetFont(font)
	level:ClearAnchors()
	level:SetAnchor(top,name,topRelative,0,0)
	level:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
	level:SetVerticalAlignment(TEXT_ALIGN_TOP)
	container.level = level
	
	local caption = container.caption or WINDOW_MANAGER:CreateControl(nil,container,CT_LABEL)
	caption:ClearAnchors()
	caption:SetAnchor(bottom,name,bottomRelative,0,0)
	caption:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
	caption:SetVerticalAlignment(TEXT_ALIGN_TOP)
	caption:SetFont(fontCaption)
	container.caption = caption

	-- Creating AvA rank icon
	local avaRankIcon =  container.avaRankIcon or WINDOW_MANAGER:CreateControl(nil,container,CT_TEXTURE)
	avaRankIcon:ClearAnchors()
	avaRankIcon:SetDimensions(iconSize,iconSize)
	avaRankIcon:SetAnchor(LEFT,name,RIGHT,5,0)
	avaRankIcon:SetColor(221/255,90/255,76/255,1)
	container.avaRankIcon = avaRankIcon
	
	-- Creating class icon
	local classIcon = container.classIcon or  WINDOW_MANAGER:CreateControl(nil,container,CT_TEXTURE)
	classIcon:ClearAnchors()
	classIcon:SetDimensions(iconSize,iconSize)
	classIcon:SetAnchor(RIGHT,level,LEFT,0,0)
	container.classIcon = classIcon
	
	-- Creating alliance icon
	local allianceIcon = container.allianceIcon or  WINDOW_MANAGER:CreateControl(nil,container,CT_TEXTURE)
	allianceIcon:SetDimensions(iconSize,iconSize)
	allianceIcon:ClearAnchors()
	allianceIcon:SetAnchor(RIGHT,name,LEFT,-5,0)
	container.allianceIcon = allianceIcon
	
	
	-- Creating veteran icon
	local veteranIcon = container.veteranIcon or WINDOW_MANAGER:CreateControl(nil,container,CT_TEXTURE)
	veteranIcon:SetDimensions(iconSize,iconSize)
	veteranIcon:ClearAnchors()
	veteranIcon:SetAnchor(LEFT,level,RIGHT,0,0)
	veteranIcon:SetTexture('/esoui/art/unitframes/target_veteranrank_icon.dds')
	container.veteranIcon = veteranIcon
	
	local monsterDiff = container.monsterDiff or WINDOW_MANAGER:CreateControl(nil,container,CT_LABEL)
	monsterDiff:ClearAnchors()
	monsterDiff:SetAnchor(TOP,name,BOTTOM,0,-nameplateCfg.fontSize/2)
	monsterDiff:SetFont(fontDifficulty)
	container.monsterDiff = monsterDiff

	container:SetDimensions(200,80)
	
	self.movableFrames.targetNameplate = container
	self.movableFrames.targetNameplate.anchorConfig = nameplateCfg.anchor
end

-- Build the text of the given bar
function CurvyHud:BuildTextValues(barName)
	local cfg 	= self.config
	local bar 	= self.bars[barName]
	local barCfg= bar.config
	local font 	= string.format('%s|%d|%s', LMP:Fetch('font',barCfg.fontName), barCfg.fontSize, 'soft-shadow-thick')
	
	-- creating the container
	local container 		= bar.textContainer or  WINDOW_MANAGER:CreateControl(nil,self.topControl,CT_CONTROL)
	bar.textContainer 		= container
	barCfg.textAnchor[2]	= GuiRoot
	container:ClearAnchors()
	container:SetAnchor(unpack(barCfg.textAnchor))
	
	-- creating label
	local text 		= container.text or WINDOW_MANAGER:CreateControl(nil, container, CT_LABEL)
	container.text 	= text
	text:SetColor(unpack(barCfg.textColor))
	text:SetFont(font)
	text:ClearAnchors()
	text:SetAnchor(CENTER)
	text:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
	text:SetVerticalAlignment(TEXT_ALIGN_CENTER)	
	text:SetText(FormatTextValues(barCfg.textFormat,1000,1000,100))
	
	container:SetDimensions(text:GetDimensions())
	
	-- Adding text container to movable frames
	self.movableFrames[barName.."Text"] 				= container
	self.movableFrames[barName.."Text"].anchorConfig 	= barCfg.textAnchor
	
	
	-- if healthBar or targetBar, creating the shield text values
	if(barName == 'healthBar' or barName == 'targetBar') then
		local shieldCfg	= (barName == 'targetBar' and cfg.targetVisuals.shield or cfg.visuals.shield)
		local frameName = (barName == 'targetBar' and "targetShieldText" or "shieldText")
		
		local shieldFont 	= string.format('%s|%d|%s', LMP:Fetch('font',shieldCfg.fontName), shieldCfg.fontSize, 'soft-shadow-thick')
		-- creating the shield text container
		local container 			= bar.shield.textContainer or  WINDOW_MANAGER:CreateControl(nil,self.topControl,CT_CONTROL)
		bar.shield.textContainer	= container
		shieldCfg.textAnchor[2]		= GuiRoot
		container:ClearAnchors()
		container:SetAnchor(unpack(shieldCfg.textAnchor))
	
		-- creating label
		local text		= container.text or WINDOW_MANAGER:CreateControl(nil, container, CT_LABEL)
		container.text 	= text
		text:SetColor(unpack(shieldCfg.textColor))
		text:SetFont(shieldFont)
		text:ClearAnchors()
		text:SetAnchor(CENTER)
		text:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
		text:SetVerticalAlignment(TEXT_ALIGN_CENTER)	
		text:SetText(FormatTextValues(shieldCfg.textFormat,1000))
		text:SetHidden(true)
		
		container:SetDimensions(text:GetDimensions())
		
		-- Adding text container to movable frames
		self.movableFrames[frameName] 				= container
		self.movableFrames[frameName].anchorConfig 	= shieldCfg.textAnchor
	end
end

-- (Re)build the interaction prompt (in fact we just move it)
function CurvyHud:BuildInteractionPrompt()
	local cfg = self.config
	-- Adjust the interact/non-interact prompt
	local container = self.interactionPrompt or WINDOW_MANAGER:CreateTopLevelWindow("CURVY_InteractionPrompt")
	cfg.interactionPrompt.anchor[2] = GuiRoot
	container:ClearAnchors()
	container:SetAnchor(unpack(cfg.interactionPrompt.anchor))
	container:SetDimensions(300,100)
	
	ZO_ReticleContainerInteract:ClearAnchors()
	ZO_ReticleContainerInteract:SetAnchor(TOP,container,TOP,0,0)
	
	ZO_ReticleContainerInteractContext:ClearAnchors()
	ZO_ReticleContainerInteractContext:SetAnchor(CENTER,ZO_ReticleContainerInteract,CENTER,0,0)
	ZO_ReticleContainerInteractContext:SetVerticalAlignment(TEXT_ALIGN_TOP)
	ZO_ReticleContainerInteractContext:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
	
	ZO_ReticleContainerInteractKeybindButton:ClearAnchors()
	ZO_ReticleContainerInteractKeybindButton:SetAnchor(TOP,ZO_ReticleContainerInteractContext,BOTTOM,0,0)
	
	ZO_ReticleContainerNonInteract:ClearAnchors()
	ZO_ReticleContainerNonInteract:SetAnchor(TOP,container,TOP,0,0)
	ZO_ReticleContainerNonInteractText:SetVerticalAlignment(TEXT_ALIGN_TOP)
	ZO_ReticleContainerNonInteractText:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
	
	self.interactionPrompt = container
	self.movableFrames.interactionPrompt = container
	self.movableFrames.interactionPrompt.anchorConfig = cfg.interactionPrompt.anchor

end

--------------
-- Updating --
--------------

-- Update the given bar with the new values
function CurvyHud:UpdateBar(barName,currentValue, maxValue, doAnim)
	-- useful locals
	local cfg		 	= self.config
	local bar 			= self.bars[barName]
	local baseTCoords 	= bar.filler.baseTextureCoords
	local baseHeight 	= bar.filler.baseHeight

	-- Calculating ratio, new height and texture height
	
	local ratio 		= math.ceil((currentValue / maxValue)*100)/100
	local newHeight 	= baseHeight * ratio
	local newTCoordTop 	= baseTCoords[4]- (ratio * (baseTCoords[4]-baseTCoords[3]))
	
	local shieldValue 	= (barName == 'targetBar' and self.targetVisuals.shield.value or  self.visuals.shield.value)
	local shieldCfg		= (barName == 'targetBar' and cfg.targetVisuals.shield or  cfg.visuals.shield)
	local shieldNewHeight
	local shieldRatio
	local shieldNewTCoordBottom
	local shieldNewTCoordTop
	
	-- Calculating ratio, new height and texture height for the shield bar
	if((barName =='healthBar' or barName == 'targetBar') and shieldCfg.show) then
		shieldRatio =  math.ceil((shieldValue / maxValue)*100)/100
		shieldNewHeight = baseHeight * shieldRatio
		shieldNewTCoordBottom = baseTCoords[3] + (shieldRatio * (baseTCoords[4]-baseTCoords[3]))
		shieldNewTCoordTop = newTCoordTop - (shieldRatio * (baseTCoords[4]-baseTCoords[3]))
	end

	doAnim =true
	if(doAnim) then
		bar:PlayBaseAnim(barName,newHeight,newTCoordTop, shieldNewHeight, shieldNewTCoordTop,shieldNewTCoordBottom)
	else
		-- if we don't use the animation, use this code instead (instant decrease)
		bar.filler:SetHeight(newHeight)
		bar.filler:SetTextureCoords(
			baseTCoords[1],
			baseTCoords[2],
			newTCoordTop,
			baseTCoords[4]
		)
		
		if((barName =='healthBar' or barName == 'targetBar') and shieldCfg.show) then
			bar.shield:SetHeight(shieldNewHeight)
			bar.shield:SetTextureCoords(
			baseTCoords[1],
			baseTCoords[2],
			shieldNewTCoordTop,
			shieldNewTCoordBottom
		)
		end
	end
		
	-- setting new colors
	-- Note : this is not part of the animation, as I don't want to add more calculation to it,
	-- As a consequence, colors will instantly goes to the 'next' value, so not that important
	local rBar = bar.config.colorLow[1]+bar.filler.rGap* ratio
	local gBar = bar.config.colorLow[2]+bar.filler.gGap* ratio
	local bBar = bar.config.colorLow[3]+bar.filler.bGap* ratio
	local aBar = bar.config.colorLow[4]+bar.filler.aGap* ratio
	
	local rBorder = bar.config.borderColorLow[1]+bar.border.rGap* ratio
	local gBorder = bar.config.borderColorLow[2]+bar.border.gGap* ratio
	local bBorder = bar.config.borderColorLow[3]+bar.border.bGap* ratio
	local aBorder = bar.config.borderColorLow[4]+bar.border.aGap* ratio
	
	-- changing colors
	bar.filler:SetColor(rBar,gBar,bBar,aBar)
	bar.border:SetColor(rBorder,gBorder,bBorder,aBorder)
end

-- Update the target nameplate
function CurvyHud:UpdateTargetNameplate()
	-- useful locals
	local cfg = self.config
	local nameplate = self.targetNameplate
	local nameplateCfg = cfg.targetNameplate
	local target
	
	-- if in move mode, we use a dummy target, i.e. set the player as target
	if(self.moveMode.enabled) then target = 'player' else target = 'reticleover' end
	
	-- Getting basic target info
	local nameText = GetUnitName(target)
	local captionText = GetUnitCaption(target)
	local levelText = GetUnitLevel(target)
	
	
	-- Getting reaction color
	local reacR,reacG,reacB = GetUnitReactionColor(target)
	
	-- Coloring name by reaction, or white (coloring by level come after, if needed)
	if(nameplateCfg.colorName == 'reaction') then 
		nameplate.name:SetColor(reacR,reacG,reacB,1)
	else
		nameplate.name:SetColor(1,1,1,1)
	end
	
	-- Checking if target has a valid level
	if(levelText ~= nil and levelText > 0) then
		-- Getting difficulty color
		local r,g,b = GetConColor(levelText,GetUnitLevel('player'))
		
		-- Coloring level
		if(nameplateCfg.colorLevel == 'difficulty') then
			nameplate.level:SetColor(r,g,b,1)
		elseif(nameplateCfg.colorLevel == 'reaction') then
			nameplate.level:SetColor(reacR,reacG,reacb,1)
		else
			nameplate.level:SetColor(1,1,1,1)
		end
			
		if(nameplateCfg.colorName == 'difficulty') then
			nameplate.name:SetColor(r,g,b,1)
		end
	else
		levelText = ''
	end
	
	-- Assigning values
	nameplate.name:SetText(nameText)
	nameplate.caption:SetText(captionText)
	nameplate.level:SetText(levelText)
	
	-- If target is veteran, show veteran icon and change level to veteran rank
	if(IsUnitVeteran(target)) then
		nameplate.veteranIcon:SetHidden(false)
		nameplate.level:SetText(GetUnitVeteranRank(target))
	else
		nameplate.veteranIcon:SetHidden(true)
	end
		
	-- Do something if target is a player
	if(IsUnitPlayer(target)) then
		
		-- Set up class, alliance and rank icons
		local classId = GetUnitClassId(target)	
		nameplate.classIcon:SetHidden(not nameplateCfg.classIcon)
		nameplate.classIcon:SetTexture(classIcons[classId])
		
		local allianceId = GetUnitAlliance(target)nameplate.allianceIcon:SetHidden(not nameplateCfg.allianceIcon)
		nameplate.allianceIcon:SetTexture(allianceIcons[allianceId])
		
		local rank = GetUnitAvARank(target)
		nameplate.avaRankIcon:SetHidden(not nameplateCfg.rankIcon)
		nameplate.avaRankIcon:SetTexture(GetAvARankIcon(rank))
	
		-- set caption to title
		nameplate.caption:SetText(GetUnitTitle(target))
		
		-- hide monster difficulty indicator
		nameplate.monsterDiff:SetHidden(true)
		
		
	else -- if not a player, hide class/alliance/ava/veteran icons
	-- also add monster difficulty indicator
		nameplate.classIcon:SetHidden(true)
		nameplate.allianceIcon:SetHidden(true)
		nameplate.avaRankIcon:SetHidden(true)
		
		nameplate.monsterDiff:SetHidden(false)
		if(GetUnitDifficulty(target) == MONSTER_DIFFICULTY_NORMAL) then
			nameplate.monsterDiff:SetColor(200,200,0,1)
			nameplate.monsterDiff:SetText("*")
		elseif(GetUnitDifficulty(target) == MONSTER_DIFFICULTY_HARD) then
			nameplate.monsterDiff:SetColor(240,100,0,1)
			nameplate.monsterDiff:SetText("* *")
		elseif(GetUnitDifficulty(target) == MONSTER_DIFFICULTY_DEADLY) then
			nameplate.monsterDiff:SetColor(200,0,0,1)
			nameplate.monsterDiff:SetText("* * *")
		else
			nameplate.monsterDiff:SetText("")
		end
	end
	
end

-- Update the text of the given bar with the new values
function CurvyHud:UpdateTextValues(barName, currentValue,maxValue)
	local bar = CurvyHud.bars[barName]
	local percentValue = string.format('%d', math.ceil((currentValue / maxValue * 100)))
	
	bar.textContainer.text:SetText(FormatTextValues(bar.config.textFormat,currentValue,maxValue,percentValue))
	if(barName =='healthBar') then
		if(self.visuals.shield.on and self.config.visuals.shield.showText) then
			bar.shield.textContainer.text:SetHidden(false)
			bar.shield.textContainer.text:SetText(FormatTextValues(self.config.visuals.shield.textFormat,self.visuals.shield.value))
		else
			bar.shield.textContainer.text:SetHidden(true)
		end
	elseif(barName =='targetBar') then
		if(self.targetVisuals.shield.on and self.config.targetVisuals.shield.showText) then
			bar.shield.textContainer.text:SetHidden(false)
			bar.shield.textContainer.text:SetText(FormatTextValues(self.config.targetVisuals.shield.textFormat,self.targetVisuals.shield.value))
		else
			bar.shield.textContainer.text:SetHidden(true)
		end
	end
end

---------------
-- Animating --
---------------

function CurvyHud.CreateBaseAnim(barName, bar)
	local filler = bar.filler
	local baseHeight = filler.baseHeight
	
	-- first we create the timeline
	local timeline = ANIMATION_MANAGER:CreateTimeline()
	filler.timeline = timeline
	
	-- then insert the animation and set its duration and easing function
	local anim = timeline:InsertAnimation(ANIMATION_CUSTOM, filler)
	filler.anim = anim
	anim:SetDuration(200)
	anim:SetEasingFunction(ZO_EaseOutCubic)
	
	-- setting some values used in the update function
	anim.height = baseHeight
	anim.tCoordTop = filler.baseTextureCoords[3]
	anim.baseTCoords = filler.baseTextureCoords
	
	-- setting the update function
	anim:SetUpdateFunction(
		function(self)
			local progress = timeline:GetProgress()
			self.height 	= self.startHeight 		- self.heightOffset		* progress
			self.tCoordTop 	= self.startTCoordTop 	- self.tCoordTopOffset	* progress
			filler:SetHeight(self.height)
			filler:SetTextureCoords(
				self.baseTCoords[1],
				self.baseTCoords[2],
				self.tCoordTop,
				self.baseTCoords[4]
			)
		end
	)

	
	-- finally, provide and easy function to play the anim
	-- requires filler and shield new heights and texture coordinates
	-- those are calculated in UpdateBar(), so call this one there
	bar.PlayBaseAnim = function(self,barName,newHeight,newTCoordTop)
		-- setting things up for the update function
		local anim = self.filler.anim
		anim.heightOffset	= anim.height - newHeight
		anim.tCoordTopOffset= anim.tCoordTop - newTCoordTop
		anim.startHeight	= anim.height
		anim.startTCoordTop = anim.tCoordTop
		
		-- if an anim is already playing we stop it
		if(	self.filler.timeline:IsPlaying()) then
			self.filler.timeline:Stop()
		end
	
		-- Playing the animation
		self.filler.timeline:PlayFromStart()
	
	end
end

function CurvyHud.CreateShieldAnim(barName,bar)
	local filler = bar.filler
	local baseHeight = filler.baseHeight
	
	-- first we create the timeline
	local timeline = ANIMATION_MANAGER:CreateTimeline()
	filler.timeline = timeline
	
	-- then insert the animation and set its duration and easing function
	local anim = timeline:InsertAnimation(ANIMATION_CUSTOM, filler)
	filler.anim = anim
	anim:SetDuration(200)
	anim:SetEasingFunction(ZO_EaseOutCubic)
	
	-- setting some values used in the update function
	anim.height = baseHeight
	anim.tCoordTop = filler.baseTextureCoords[3]
	anim.baseTCoords = filler.baseTextureCoords
	

	-- setting some shield values first
	anim.shieldHeight 		= 0
	anim.shieldTCoordBottom = filler.baseTextureCoords[3]
	anim.shieldTCoordTop	= anim.shieldTCoordBottom
	anim:SetUpdateFunction(
		function(self)
			local progress = timeline:GetProgress()
			self.height 	= self.startHeight 		- self.heightOffset		* progress
			self.tCoordTop 	= self.startTCoordTop 	- self.tCoordTopOffset	* progress
			filler:SetHeight(self.height)
			filler:SetTextureCoords(
				self.baseTCoords[1],
				self.baseTCoords[2],
				self.tCoordTop,
				self.baseTCoords[4]
			)
			self.shieldHeight 		= self.shieldStartHeight 		- self.shieldHeightOffset		* progress
			self.shieldTCoordBottom = self.shieldStartTCoordBottom 	- self.shieldTCoordBottomOffset	* progress
			self.shieldTCoordTop 	= self.shieldStartTCoordTop 	- self.shieldTCoordTopOffset	* progress
			
			bar.shield:SetHeight(self.shieldHeight)
			bar.shield:ClearAnchors()
			if(self.shieldTCoordTop <self.baseTCoords[3]) then
				bar.shield:SetAnchor(unpack(bar.shield.baseAnchor))
				bar.shield:SetTextureCoords(
					self.baseTCoords[1],
					self.baseTCoords[2],
					self.baseTCoords[3],
					self.shieldTCoordBottom
				)
			else
				bar.shield:SetAnchor(BOTTOMLEFT,filler,TOPLEFT,0,0)
				bar.shield:SetTextureCoords(
					self.baseTCoords[1],
					self.baseTCoords[2],
					self.shieldTCoordTop,
					self.tCoordTop
				)
			end
		
		end
	)	
	
	-- finally, provide and easy function to play the anim
	-- requires filler and shield new heights and texture coordinates
	-- those are calculated in UpdateBar(), so call this one there
	bar.PlayBaseAnim = function(self,barName,newHeight,newTCoordTop, shieldNewHeight, shieldNewTCoordTop,shieldNewTCoordBottom)
		-- setting things up for the update function
		local anim = self.filler.anim
		anim.heightOffset	= anim.height - newHeight
		anim.tCoordTopOffset= anim.tCoordTop - newTCoordTop
		anim.startHeight	= anim.height
		anim.startTCoordTop = anim.tCoordTop
		anim.shieldHeightOffset			= anim.shieldHeight - shieldNewHeight
		anim.shieldTCoordBottomOffset	= anim.shieldTCoordBottom - shieldNewTCoordBottom
		anim.shieldTCoordTopOffset		= anim.shieldTCoordTop - shieldNewTCoordTop
		anim.shieldStartHeight			= anim.shieldHeight
		anim.shieldStartTCoordBottom	= anim.shieldTCoordBottom
		anim.shieldStartTCoordTop		= anim.shieldTCoordTop
	
		
		-- if an anim is already playing we stop it
		if(	self.filler.timeline:IsPlaying()) then
			self.filler.timeline:Stop()
		end
	
		-- Playing the animation
		self.filler.timeline:PlayFromStart()
	
	end
end

function CurvyHud.CreateRegenAnim(barName,bar, fillerStepSize)
	local cfg = CurvyHud.config
	local regen = bar.regen
	local filler = bar.filler
	local fillerHeight = filler.baseHeight
	local fillerTCoords = bar.filler.baseTextureCoords
	local regenTCoords = bar.regen.baseTextureCoords
	
	local timeline = regen.timeline or ANIMATION_MANAGER:CreateTimeline()
	regen.timeline = timeline
	timeline:SetPlaybackType(ANIMATION_PLAYBACK_LOOP,100000)
	
	local anim = regen.anim or timeline:InsertAnimation(ANIMATION_CUSTOM, regen)
	regen.anim = anim
	
	local fillerTHeight = fillerTCoords[4]-fillerTCoords[3]
	
	anim.startCoord = fillerHeight
	anim.startTCoord= fillerTCoords[4]
	anim.stopCoord 	= fillerHeight/2 -30
	anim.stopTCoord = anim.stopCoord / cfg.barHeight	
	anim.height = fillerHeight/2 -30
	
	anim.arrowSpan = math.min(fillerHeight/6,anim.height)
	anim.arrowTSpan = math.min(fillerTHeight/6,anim.height/ cfg.barHeight)
	
	anim.baseAnchor = {BOTTOMLEFT,bar,BOTTOMLEFT,0,-fillerStepSize}
	
	regen:SetAnchor(unpack(anim.baseAnchor))
	-- this function is used to set a time between each loop (the loopOffset value, in ms)
	-- anim.baseDuration is equal to the actual time, in ms, the animation takes to run its course (i.e. from starting point to top point)
	function anim:SetLoopOffset(offset)
		self.loopOffset = offset
		self.offsetRatio = (self.baseDuration + self.loopOffset) / self.baseDuration
		self:SetDuration(self.baseDuration*self.offsetRatio)
	end
	
	
	anim.baseDuration = 800
	anim:SetLoopOffset(0)
	anim.expanding = true
	
	anim:SetUpdateFunction(
		function(self)
			local progress = timeline:GetProgress()*self.offsetRatio
			if(progress >= 1) then
				-- d(tostring(progress*100)..'% - Resetting')
				regen:SetHeight(0)
				regen:SetTextureCoords(
					regenTCoords[1],
					regenTCoords[2],
					self.startTCoord,
					self.startTCoord
				)
				regen:ClearAnchors()
				regen:SetAnchor(unpack(self.baseAnchor))
				-- if(self.lastUpdate ~= nil and GetGameTimeMilliseconds() > self.lastUpdate + 20000) then
					-- self:SetLoopOffset(1500)
					-- CurvyHud.visuals.regen[barName] = false
					-- self.lastUpdate = nil
					-- CurvyHud:FadePlayerBar(barName)
					-- d('pouet')
				-- end
				self.expanding = true
			else
				local actualSpan = math.ceil(regen:GetHeight())
				local tCoordTop = self.startTCoord-(self.stopTCoord+self.arrowTSpan)*progress
				local tCoordBottom = tCoordTop + self.arrowTSpan
				
				local yOffset = (self.stopCoord+self.arrowSpan)*progress -self.arrowSpan
				
				if(actualSpan < self.arrowSpan and self.expanding) then
					-- d(tostring(progress*100)..'% - Expanding, actualSpan = ' .. actualSpan..", expanding = "..tostring(self.expanding))
					regen:SetTextureCoords(
						regenTCoords[1],
						regenTCoords[2],
						tCoordTop,
						self.startTCoord
					)
					regen:SetHeight(math.min((self.stopCoord+self.arrowSpan)*progress),self.arrowSpan)
					regen:ClearAnchors()
					regen:SetAnchor(unpack(self.baseAnchor))
				elseif(actualSpan >= self.arrowSpan and tCoordTop >= self.startTCoord-self.stopTCoord) then
					self.expanding = false
					-- d(tostring(progress*100)..'% - Rising, actualSpan = ' .. actualSpan..", expanding = "..tostring(self.expanding))	
					regen:SetHeight(self.arrowSpan)
					regen:SetTextureCoords(
						regenTCoords[1],
						regenTCoords[2],
						tCoordTop,
						tCoordBottom 
					)
					regen:ClearAnchors()
					regen:SetAnchor(
						self.baseAnchor[1],
						self.baseAnchor[2],
						self.baseAnchor[3],
						self.baseAnchor[4],
						self.baseAnchor[5]-yOffset
					)
				else
					self.expanding = false
					-- d(tostring(progress*100)..'% - Diminishing, actualSpan = ' .. actualSpan..", expanding = "..tostring(self.expanding))
					regen:SetTextureCoords(
						regenTCoords[1],
						regenTCoords[2],
						self.startTCoord-self.stopTCoord,
						tCoordBottom 
					)
					regen:SetHeight(self.stopCoord - yOffset )
					regen:ClearAnchors()
					regen:SetAnchor(
						self.baseAnchor[1],
						self.baseAnchor[2],
						self.baseAnchor[3],
						self.baseAnchor[4],
						self.baseAnchor[5]-yOffset
					)
				end
			end			
		end
	)
end

function CurvyHud.CreateDegenAnim(barName,bar, fillerStepSizeT)
	local cfg = CurvyHud.config
	local degen = bar.degen
	local filler = bar.filler
	local fillerHeight = filler.baseHeight
	local fillerTCoords = bar.filler.baseTextureCoords
	local degenTCoords = bar.degen.baseTextureCoords
	
	local timeline = degen.timeline or ANIMATION_MANAGER:CreateTimeline()
	degen.timeline = timeline
	timeline:SetPlaybackType(ANIMATION_PLAYBACK_LOOP,100000)
	
	local anim = degen.anim or timeline:InsertAnimation(ANIMATION_CUSTOM, degen)
	degen.anim = anim
	
	local fillerTHeight = fillerTCoords[4]-fillerTCoords[3]
	
	anim.startCoord = fillerHeight
	anim.startTCoord= fillerTCoords[3]
	anim.stopCoord 	= fillerHeight/2 -30
	anim.stopTCoord = anim.stopCoord / cfg.barHeight	
	anim.height 	= fillerHeight/2 -30
	
	anim.arrowSpan = math.min(fillerHeight/6,anim.height)
	anim.arrowTSpan = math.min(fillerTHeight/6,anim.height/ cfg.barHeight)
	
	anim.baseAnchor = {TOPLEFT,bar,TOPLEFT,0,fillerStepSizeT}
	
	degen:SetAnchor(unpack(anim.baseAnchor))
	-- this function is used to set a time between each loop (the loopOffset value, in ms)
	-- anim.baseDuration is equal to the actual time, in ms, the animation takes to run its course (i.e. from starting point to top point)
	function anim:SetLoopOffset(offset)
		self.loopOffset = offset
		self.offsetRatio = (self.baseDuration + self.loopOffset) / self.baseDuration
		self:SetDuration(self.baseDuration*self.offsetRatio)
	end
	
	
	anim.baseDuration = 800
	anim:SetLoopOffset(0)
	anim.expanding = true
	
	anim:SetUpdateFunction(
		function(self)
			local progress = timeline:GetProgress()*self.offsetRatio
			if(progress >= 1) then
				-- d(tostring(progress*100)..'% - Resetting')
				degen:SetHeight(0)
				degen:SetTextureCoords(
					degenTCoords[1],
					degenTCoords[2],
					self.startTCoord,
					self.startTCoord
				)
				degen:ClearAnchors()
				degen:SetAnchor(unpack(self.baseAnchor))
				-- if(self.lastUpdate ~= nil and GetGameTimeMilliseconds() > self.lastUpdate + 20000) then
					-- self:SetLoopOffset(1500)
					-- CurvyHud.visuals.degen[barName] = false
					-- self.lastUpdate = nil
					-- CurvyHud:FadePlayerBar(barName)
					-- d('pouet')
				-- end
				self.expanding = true
			else
				local actualSpan 	= math.ceil(degen:GetHeight())
				local tCoordBottom 	= self.startTCoord+(self.stopTCoord+self.arrowTSpan)*progress
				local tCoordTop 	= tCoordBottom - self.arrowTSpan
				
				local yOffset = (self.stopCoord+self.arrowSpan)*progress -self.arrowSpan
				if(actualSpan < self.arrowSpan and self.expanding) then
					-- d(tostring(progress*100)..'% - Expanding, actualSpan = ' .. actualSpan..", expanding = "..tostring(self.expanding))
					degen:SetTextureCoords(
						degenTCoords[1],
						degenTCoords[2],
						self.startTCoord,
						tCoordBottom
					)
					degen:SetHeight(math.min((self.stopCoord+self.arrowSpan)*progress),self.arrowSpan)
					degen:ClearAnchors()
					degen:SetAnchor(unpack(self.baseAnchor))
				elseif(actualSpan >= self.arrowSpan and tCoordBottom <= self.startTCoord+self.stopTCoord) then
					self.expanding = false
					-- d(tostring(progress*100)..'% - Rising, actualSpan = ' .. actualSpan..", expanding = "..tostring(self.expanding))	
					degen:SetHeight(self.arrowSpan)
					degen:SetTextureCoords(
						degenTCoords[1],
						degenTCoords[2],
						tCoordTop,
						tCoordBottom 
					)
					degen:ClearAnchors()
					degen:SetAnchor(
						self.baseAnchor[1],
						self.baseAnchor[2],
						self.baseAnchor[3],
						self.baseAnchor[4],
						self.baseAnchor[5]+yOffset
					)
				else
					self.expanding = false
					-- d(tostring(progress*100)..'% - Diminishing, actualSpan = ' .. actualSpan..", expanding = "..tostring(self.expanding))
					degen:SetTextureCoords(
						degenTCoords[1],
						degenTCoords[2],
						tCoordTop,
						self.startTCoord+self.stopTCoord
					)
					degen:SetHeight(self.stopCoord - yOffset )
					degen:ClearAnchors()
					degen:SetAnchor(
						self.baseAnchor[1],
						self.baseAnchor[2],
						self.baseAnchor[3],
						self.baseAnchor[4],
						self.baseAnchor[5]+yOffset
					)
				end
			end			
		end
	)
end