--------------------------------------------------------------	
-- /Events.lua												--
-- Contains main events handlers.							--
-- Function names are self explanatory.						--
-- Buff events handlers are located in Visuals/Events.lua	--
--------------------------------------------------------------

function CurvyHud.OnTargetChanged()
	-- hiding default target frame if needed
	CurvyHud:ToggleDefaultTargetFrame()
	-- initializing the target and fading to appropriate opacity
	CurvyHud:InitTargetBar()
	CurvyHud:FadeTargetBar()
end

function CurvyHud.OnReticleHidden(event,hidden)
	CurvyHud:HideHud(hidden)
end

function CurvyHud.OnPowerUpdated(event,unitTag, powerIndex, powerType, powerValue, powerMax, powerEffectiveMax )
	-- Player 
	if ( unitTag == 'player' ) then
		local barName = CurvyHud.powerTypeBarMapping[powerType]
		local bar = CurvyHud.bars[barName]
		local visuals = CurvyHud.visuals
		
		if(bar) then
			if(bar.config.show or bar.config.showText) then
				if(bar.config.show) then
					CurvyHud:UpdateBar(barName,powerValue,powerEffectiveMax)
					
					if(barName == "werewolfBar" and CurvyHud.state.werewolf and powerValue <= 0) then
						CurvyHud.OnWerewolfStateChanged(nil,false)
					end
				end 
				if(bar.config.showText) then 
					CurvyHud:UpdateTextValues(barName, powerValue, powerEffectiveMax)
				end	
				-- fading to appropriate opacity when out of combat
				if(not CurvyHud.state.inCombat)then
					CurvyHud:FadePlayerBar(barName,powerValue,powerEffectiveMax)
				end
			end
		end

		
		-- Ultimate
		--elseif ( powerType == POWERTYPE_ULTIMATE ) then
			-- TODO
		
	-- Target
	elseif ( unitTag == 'reticleover' ) then
		-- Health
		if (powerType == POWERTYPE_HEALTH) then
			if(CurvyHud.config.targetBar.show) then
				CurvyHud:UpdateBar('targetBar',powerValue,powerEffectiveMax)
			end
			if(CurvyHud.config.targetBar.showText) then
				CurvyHud:UpdateTextValues('targetBar', powerValue, powerEffectiveMax) 
			end
		end
	-- Siege weaponry
	elseif ( unitTag == 'controlledsiege' ) then
		if (powerType == POWERTYPE_HEALTH) then
			if(CurvyHud.config.siegeBar.show) then
				CurvyHud:UpdateBar('siegeBar',powerValue,powerEffectiveMax)
			end
			if(CurvyHud.bars.siegeBar.config.showText)then 
				CurvyHud:UpdateTextValues('siegeBar', powerValue, powerEffectiveMax)
			end
		end
	end
end

function CurvyHud.OnCombatStateChanged(event, inCombat)
	local cfg = CurvyHud.config
	
	CurvyHud.state.inCombat = inCombat
	CurvyHud:ShowHud()
end

function CurvyHud.OnMountedStateChanged(event, mounted)
	CurvyHud.state.mounted = mounted
	if(mounted) then
		CurvyHud.bars.mountBar:SetHidden(not CurvyHud.config.mountBar.show)
		CurvyHud.bars.mountBar.textContainer:SetHidden(not CurvyHud.config.mountBar.showText)
	else
		CurvyHud.bars.mountBar:SetHidden(true)
		CurvyHud.bars.mountBar.textContainer:SetHidden(true)
	end
end

-- Currently bugged : the event doesn't always fire at the end of the transformation
function CurvyHud.OnWerewolfStateChanged(event, werewolf)
	CurvyHud.state.werewolf = werewolf
	if(werewolf) then
		CurvyHud.bars.werewolfBar:SetHidden(not CurvyHud.config.werewolfBar.show)
		CurvyHud.bars.werewolfBar.textContainer:SetHidden(not CurvyHud.config.werewolfBar.showText)
	else
		CurvyHud.bars.werewolfBar:SetHidden(true)
		CurvyHud.bars.werewolfBar.textContainer:SetHidden(true)
	end
end

function CurvyHud.OnSiegeStateChanged(event)
	CurvyHud.state.siege = IsPlayerControllingSiegeWeapon()
	if(CurvyHud.state.siege) then
		CurvyHud.bars.siegeBar:SetHidden(not CurvyHud.config.siegeBar.show)
		CurvyHud.bars.siegeBar.textContainer:SetHidden(not CurvyHud.config.siegeBar.showText)
	else
		CurvyHud.bars.siegeBar:SetHidden(true)
		CurvyHud.bars.siegeBar.textContainer:SetHidden(true)
	end
end

function CurvyHud.OnPlayerDead(event)
	CurvyHud.state.playerDead = true
	
	-- if player is dead, HUD should be hidden if death recap is shown
	CurvyHud:HideHudForDeathRecap(true)
end

function CurvyHud.OnPlayerAlive(event)
	CurvyHud.state.playerDead = false
	
	-- we deactivate the death recap visibility mode
	CurvyHud:HideHudForDeathRecap(false)
	
	-- also, update all bars (because it don't always update by itself, don't know why) and show HUD for good
	CurvyHud:InitPlayerBars()
	CurvyHud.OnReticleHidden(0,IsReticleHidden())
end
