function CurvyHud:InitializeState()
	-- initializing several player states
	self.state.hasTarget = self:HasTarget()
	self.state.inCombat = IsUnitInCombat('player')
	self.state.mounted = IsMounted('player')
	self.state.siege = IsPlayerControllingSiegeWeapon()
	self.state.werewolf = CurvyHud:IsInWerewolfState()
	self.state.playerDead = IsUnitDead('player')
	
	-- initializing player and target bars
	self:InitPlayerBars()
	self:InitTargetBar()	
	
	-- initializing player and target visuals
	self:InitVisuals('player')
	
	
	-- finally, show HUD
	self:ShowHud()
end

function CurvyHud:InitPlayerBars()
	-- filling up all the player bars
	for barName, bar in pairs(self.bars) do
		if(bar and barName ~= 'targetBar') then
			-- getting the resource and bar control
			local resource = {GetUnitPower('player',self.barPowertypeMapping[barName])}
			local barCfg = self.config[barName]
			-- fill up the bar only if shown, else we just hide it
			if(barCfg.show or barCfg.showText) then
				if((barName ~= "mountBar" or (barName == "mountBar" and self.state.mounted)) and
					(barName ~= "siegeBar" or (barName == "siegeBar" and self.state.siege)) and
					(barName ~= "werewolfBar" or (barName == "werewolfBar" and self.state.werewolf)) 
				) then
					self:UpdateBar(barName,resource[1],resource[3])
					self:UpdateTextValues(barName,resource[1],resource[3])
					bar:SetHidden(not barCfg.show)
					bar.textContainer:SetHidden(not barCfg.showText)
				else
					bar:SetHidden(true)
					bar.textContainer:SetHidden(true)
				end
			else
				bar:SetHidden(true)
				bar.textContainer:SetHidden(true)
			end
		end
	end
	
	-- set things up for player-is-dead state, i.e. death recap visibility mode
	if(self.state.playerDead) then
		CurvyHud:HideHudForDeathRecap(true)
	end
end

function CurvyHud:InitTargetBar()
	-- First, check for a target. In move mode, player is always the target to allow proper display of the target related controls
	self.state.hasTarget = self.moveMode.enabled or self:HasTarget()
	
	self:ResetTargetVisuals()
	if(self.state.hasTarget) then
		-- fill target bar and show it, if needed
		if(self.config.targetBar.show or self.config.targetBar.showText) then
			local health
			if(self.moveMode.enabled ) then
				health = {GetUnitPower('player', POWERTYPE_HEALTH)}
			else
				health = {GetUnitPower('reticleover', POWERTYPE_HEALTH)}
			end
			
			self:UpdateBar('targetBar',health[1],health[3])	
			self:UpdateTextValues('targetBar',health[1],health[3])	
			
			self.bars.targetBar:SetHidden(not self.config.targetBar.show)
			self.bars.targetBar.textContainer:SetHidden(not self.config.targetBar.showText)
			
			
			self:InitVisuals('reticleover')
		end
		-- showing nameplate if needed
		if(self.config.targetNameplate.show) then
			self:UpdateTargetNameplate()
			self.targetNameplate:SetHidden(false)	
		end
		
	else -- no target, hiding everything target-related
		self.bars.targetBar:SetHidden(true)
		self.targetNameplate:SetHidden(true)
		self.bars.targetBar.textContainer:SetHidden(true)
	end	
end

function CurvyHud:InitVisuals(unit)
	local visuals = {GetAllUnitAttributeVisualizerEffectInfo(unit)}
	
	local stop = false
	local i =1
	local updateFunc
	
	if(unit == 'player') then
		updateFunc = function(...) self:UpdatePlayerVisual(...) end
	elseif(unit == 'reticleover') then
		updateFunc = function(...) self:UpdateTargetVisual(...) end
	end
	
	while not stop do
		if(visuals[i] ~= nil) then
			-- if(unit == 'player') then
				-- self:UpdatePlayerVisual(true,visuals[i],visuals[i+1],visuals[i+2],visuals[i+4],visuals[i+5])
			-- elseif(unit == 'reticleover') then
				-- self:UpdateTargetVisual(true,visuals[i],visuals[i+1],visuals[i+2],visuals[i+4],visuals[i+5])
			-- end
			updateFunc(true,visuals[i],visuals[i+1],visuals[i+2],visuals[i+4],visuals[i+5])
			i=i+6
		else
			stop = true
		end
	end
	
end

-- Check for a valid target. Return true if target is valid.
function CurvyHud:HasTarget()
	local target = GetUnitName('reticleover')
	if(target == '' or target ==nil) then
		return false
	else
		if( GetUnitLevel('reticleover') ~= 1 ) then
			return true
		else
			local critters = { 
				"Butterfly","Lizard","Rat","Snake","Pony Guar","Frog","Squirrel","Rabbit",
				"Deer","Cat","Pig","Sheep","Antelope","Wasp","Monkey","Fleshflies","Centipede",
				"Chicken","Torchbug","Spider","Scorpion","Goat","Scrib","Scuttler",	"Fox"
			}
			for i = 1, #critters do 
				if target == critters[i] then return false end
			end
		end
	end
end

-- Check for werewolf state
function CurvyHud:IsInWerewolfState()
	local resource = {GetUnitPower('player',POWERTYPE_WEREWOLF)}
	
	if(resource[1] <= 0) then
		return false
	else
		return true
	end
end