local LAM = LibStub("LibAddonMenu-2.0")
local LMP = LibStub('LibMediaProvider-1.0')
local L = CurvyHud:GetLoc()

local somethingChanged = false

local function GetLMPFonts()
	local result = {}
	local i = 1
	for k,v in pairs(LMP.MediaTable.font) do
		result[i] = k
		i=i+1
	end
	return result
end

-- Create a list of the saved presets, replacing defaults config IDs by their localized,display names
local function SavedPresets()
	local result = {}
	local i = 1
	local defaultConfigs = CurvyHud.defaultConfigs
	local tmp = {}
	-- load default presets first, adding their localized names to the list
	
	for index,v in ipairs(defaultConfigs) do
		result[i] = L.defaultConfig..index
		i=i+1
	end
	
	-- then add user presets in alphabetical order
	for k,v in pairs(CurvyHud.varsAccount.userConfigs) do
		table.insert(tmp,k)
	end
	table.sort(tmp)
	for j, k in ipairs(tmp) do
		result[i] = k
		i = i+1
	end
	return result
end

local function InUserPresets(name)
	for k,v in pairs(CurvyHud.varsAccount.userConfigs) do
		if k == name then	
			return true
		end	
	end
	return false
end

local function IsPresetDefault(presetName)
	if string.sub(presetName,1,1) == "*" then
		return true
	else
		return false
	end
end

local function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

-- Setting dropdown list for bars
local dropBarList = {
	[L.Health]	= 'healthBar',
	[L.Magicka] = 'magickaBar',
	[L.Stamina] = 'staminaBar',
	[L.Target]	= 'targetBar',
	[L.Mount] 	= 'mountBar',
	[L.Siege] 	= 'siegeBar',
	[L.Werewolf]= 'werewolfBar',
}
	
--settting dropdown list for positions
local posList = {
	['left'] = L.Left,
	['right'] = L.Right
}


local mainPanel
local lastAddedControl
local headerColor = "|cF0C300"

local function AddControl(data)
	return LAMCreateControl[data.type](mainPanel,data)
end

local function AddControlToSub(submenu,data)
	return LAMCreateControl[data.type](submenu,data)
end

function CurvyHud:InitializeMenu()
	
	local panelData = {
		type = 'panel',
		name = CurvyHud.name ,
		displayName = "|cFF0400CurvyHUD",
		author = "Niocwy",
		version = "1.1",
	}
	mainPanel = LAM:RegisterAddonPanel("CurvyHudOptions", panelData)
	
	
	
	self:InitializeConfigManager()
	self:InitializeMoveModeSettings()
	self:InitializeGeneralSettings()
	self:BuildTargetNameplateSettings()
	self:InitializeBarSettings()
	self:InitializeVisualsSettings()
	self:InitializeTargetVisualsSettings()
	
end

function CurvyHud:InitializeConfigManager()
	local selectedPreset = ""
	local editNewText = ""
	
	-- early definintion of some oontrols (for use in embedded functions)
	local btnSave = {}
	btnSave.warning = {}
	btnSave.warning.tooltipText = ""
	btnSave.button = {}
	
	local btnLoad = {}
	btnLoad.warning = {}
	btnLoad.warning.tooltipText = ""
	btnLoad.button = {}
	
	local dropLoad
	
	local editNew = {}
	editNew.edit = {}
	
	local btnNew = {}	
	btnNew.warning = {}
	btnNew.warning.tooltipText = {}
	btnNew.button = {}
	
	------------
	-- Header --
	------------
	local header = AddControl({
		type = "header",
		name = headerColor..L.cfgManager,
	})
	header:SetAnchor(TOPLEFT)
	
	-----------------
	-- Description --
	-----------------
	local desc = AddControl({
		type = "description",
		text = L.cfgText,
	})
	desc:SetAnchor(TOPLEFT,header,BOTTOMLEFT,0,10)
	
	-------------------
	-- Dropdown menu --
	-------------------
	dropLoad = AddControl({
		type = "dropdown",
		choices = SavedPresets(),
		getFunc = function()
			return selectedPreset
		end,
		setFunc = function(arg)
			selectedPreset = arg
			if(not IsPresetDefault(selectedPreset)) then
				btnSave.warning:SetAlpha(1)
				btnSave.warning.tooltipText = L.cfgBtnSaveWarning
				btnSave.button:SetState(BSTATE_NORMAL)
			else
				btnSave.warning:SetAlpha(0)
				btnSave.warning.tooltipText = ''
				btnSave.button:SetState(BSTATE_DISABLED)
			end
			
			btnLoad.warning:SetAlpha(1)
			btnLoad.warning.tooltipText = L.cfgBtnLoadWarning
			btnLoad.button:SetState(BSTATE_NORMAL)
		end,
	})
	dropLoad:SetWidth(180)
	dropLoad:SetAnchor(TOPLEFT,desc,BOTTOMLEFT,0,10)

	dropLoad.combobox:ClearAnchors()
	dropLoad.combobox:SetAnchor(CENTER)
	dropLoad.combobox:SetWidth(180)
	
	-----------------
	-- Load button --
    -----------------
	btnLoad = AddControl({
		type = "button",
		name = L.cfgBtnLoad,
		warning = "",
		func = function()
			-- if the selected preset is a default config, get its ID instead of localized,display name
			if(IsPresetDefault(selectedPreset)) then
				self.config = deepcopy(self.defaultConfigs[tonumber(string.sub(selectedPreset,-1,-1))])
			else
				self.config = deepcopy(self.varsAccount.userConfigs[selectedPreset])
			end	
			
			self.vars.config = self.config
			ReloadUI()
		end,
	})
	btnLoad:SetAnchor(LEFT,dropLoad.combobox,RIGHT,0, 0)
	btnLoad:SetWidth(150)
	
	btnLoad.button:SetWidth(150)
	btnLoad.button:ClearAnchors()
	btnLoad.button:SetAnchor(CENTER)
	-- On initialization, no preset is selected, that means there's no need to load,
	-- so we deactivate the button and hide the warning
	btnLoad.button:SetState(BSTATE_DISABLED)
	btnLoad.warning.tooltipText = ""
	btnLoad.warning:SetAlpha(0)
	btnLoad.warning:ClearAnchors()
	btnLoad.warning:SetAnchor(LEFT,btnLoad.button,RIGHT,-5,0)
	
	-----------------
	-- Save button --
    -----------------
	btnSave = AddControl({
		type = "button",
		name = L.cfgBtnSave,
		warning = "",
		func = function()
			-- we can save only if the selected preset is not a default one
			if(not IsPresetDefault(selectedPreset)) then
				self.varsAccount.userConfigs[selectedPreset] = deepcopy(self.config)
				
				btnLoad.warning:SetAlpha(0)
				btnLoad.warning.tooltipText = ''
				btnLoad.button:SetState(BSTATE_DISABLED)
				
				btnSave.warning:SetAlpha(0)
				btnSave.warning.tooltipText = ''
				btnSave.button:SetState(BSTATE_DISABLED)
				
				somethingChanged = false
			end
		end,
	})
	btnSave:SetAnchor(LEFT,btnLoad,RIGHT,10,0)
	btnSave:SetWidth(150)
	
	btnSave.button:SetWidth(150)
	btnSave.button:ClearAnchors()
	btnSave.button:SetAnchor(CENTER)
	
	-- On initialization, no preset is selected that means there's no need to save 
	-- so we deactivate the button and hide the warning
	btnSave.button:SetState(BSTATE_DISABLED)
	btnSave.warning.tooltipText = ''
	btnSave.warning:SetAlpha(0)
	btnSave.warning:ClearAnchors()
	btnSave.warning:SetAnchor(LEFT,btnSave.button,RIGHT,-5,0)
	
	-----------------------------
	-- New preset description  --
	-----------------------------
	newDesc = AddControl({
		type = "description",
		text = L.cfgTextNew,
	})
	newDesc:SetAnchor(TOPLEFT,dropLoad,BOTTOMLEFT,0,10)
	
	------------------------
	-- New Preset editbox --
	------------------------
	editNew = AddControl({ 
		type = 'editbox',
		isMultiline = false,
		getFunc = function()
			return ""
		end,
		setFunc = function(arg)
			editNewText = arg
			if(editNewText ~= nil and editNewText ~= '') then
				local goodNname, x = string.gsub(editNewText,"[^a-zA-Z0-9_-]","")
				if(x > 0) then	
					btnNew.button:SetState(BSTATE_DISABLED)
					btnNew.warning.tooltipText =  L.cfgBtnNewWarningFormat
					btnNew.warning:SetAlpha(1)
				elseif(InUserPresets(editNewText)) then
					btnNew.button:SetState(BSTATE_DISABLED)
					btnNew.warning.tooltipText =  L.cfgBtnNewWarningExisting
					btnNew.warning:SetAlpha(1)
				else
					btnNew.button:SetState(BSTATE_NORMAL)
					btnNew.warning.tooltipText =  ''
					btnNew.warning:SetAlpha(0)
				end
			end
		end
	})
	editNew:SetAnchor(TOPLEFT,newDesc,BOTTOMLEFT,0,10)
	
	editNew.label:ClearAnchors()
	editNew.bg:ClearAnchors()
	editNew.label:SetAnchor(TOPLEFT)
	editNew.bg:SetAnchor(TOPLEFT)
	
	------------------------
	-- New Preset button  --
	------------------------
	btnNew = AddControl({
		type = 'button',
		name = L.cfgBtnNew,
		tooltip = L.cfgBtnNewTooltip,
		func = function ()
			if(not InUserPresets(editNewText)  and editNewText ~= nil and editNewText ~= '') then
				self.varsAccount.userConfigs[editNewText] = deepcopy(self.config)
				editNew.editbox:SetText('')
				btnNew.button:SetState(BSTATE_DISABLED)
				btnNew.warning:SetAlpha(0)
				btnNew.warning.tooltipText = ''
				
				dropLoad:UpdateChoices(SavedPresets())
				
				somethingChanged = false
			end
		end,
	})
	btnNew:SetWidth(180)
	btnNew:SetAnchor(TOPRIGHT,newDesc,BOTTOMRIGHT,-15,10)
	
	btnNew.button:ClearAnchors()
	btnNew.button:SetAnchor(RIGHT)
	btnNew.button:SetState(BSTATE_DISABLED)	
	
	btnNew.warning = WINDOW_MANAGER:CreateControlFromVirtual('CurvyHud_Cfg_BtnNewWarningIcon', btnNew, "ZO_Options_WarningIcon")
	btnNew.warning:SetAnchor(LEFT, button, RIGHT, -5, 0)
	btnNew.warning.tooltipText = ''
	btnNew.warning:SetAlpha(0)
	
	--
	lastAddedControl = editNew
end

function CurvyHud:InitializeMoveModeSettings()
	local cfg = self.config
	local btnDisable
	local btnEnable
	
	------------
	-- Header --
	------------
	local header = AddControl({
		type = 'header',
		name = headerColor..L.MoveModeHeader
	})
	header:SetAnchor(TOPLEFT,lastAddedControl,BOTTOMLEFT,0,10)
	
	local desc = AddControl({
		type = 'description',
		text = L.MoveModeText,
	})
	desc:SetAnchor(TOPLEFT,header,BOTTOMLEFT,0, 10)
	
	-------------------
	-- Unlock Button --
	-------------------
	btnEnable = AddControl({
		type = 'button',
		name = L.MoveModeBtnEnable,
		func = function()
			self:ToggleMoveFramesMode(true)
			btnEnable.button:SetState(BSTATE_DISABLED)
			btnDisable.button:SetState(BSTATE_NORMAL)
		end,
		reference = 'CurvyHudOptions_EnableMoveBtn' -- referencing for use in MoveMode.lua
	})
	btnEnable:SetWidth(255)
	btnEnable:SetAnchor(TOPLEFT,desc,BOTTOMLEFT,0,10)
	btnEnable.button:ClearAnchors()
	btnEnable.button:SetAnchor(CENTER)
	
	btnDisable = AddControl({
		type = 'button',
		name = L.MoveModeBtnDisable, 
		func = function()
			self:ToggleMoveFramesMode(false)
			btnEnable.button:SetState(BSTATE_NORMAL)
			btnDisable.button:SetState(BSTATE_DISABLED)
		end,
		reference = 'CurvyHudOptions_DisableMoveBtn' -- referencing for use in MoveMode.lua
	})
	btnDisable:SetWidth(255)
	btnDisable:SetAnchor(TOPRIGHT,desc,BOTTOMRIGHT,0,10)
	btnDisable.button:ClearAnchors()
	btnDisable.button:SetAnchor(CENTER)
	btnDisable.button:SetState(BSTATE_DISABLED)
	
	
	
	-- -- LAM:AddCheckbox(menu, 'CurvyHud_MoveMode_LockContainers', L.MoveModeLockContainer, nil,
		-- -- function()
			-- -- return self.moveMode.lockContainers
		-- -- end,
		-- -- function(arg)
			-- -- if(arg ~= self.moveMode.lockContainers) then
				 -- -- self.moveMode.lockContainers = arg
			-- -- end
		-- -- end
	-- -- )
	
	local chkLockText = AddControl({ 
		type = 'checkbox',
		name = L.MoveModeLockTextValues,
		tooltip = L.MoveModeLockTextValuesTooltip,
		getFunc = function()
			return self.moveMode.lockTextValues
		end,
		setFunc = function(arg)
			if(arg ~= self.moveMode.lockTextValues) then
				self.moveMode.lockTextValues = arg
			end
		end
	})
	chkLockText:SetAnchor(TOPLEFT,btnEnable,BOTTOMLEFT,0,10)
	
	--
	lastAddedControl = chkLockText
end

function CurvyHud:InitializeGeneralSettings()
	local cfg = self.config
	
	local subMenu = AddControl({
		type = 'submenu',
		name = headerColor..L.GeneralSettings,
	})
	subMenu:SetAnchor(TOPLEFT,lastAddedControl,BOTTOMLEFT,0,10)
	
	local chkHidePlayer = AddControlToSub(subMenu,{
		type = 'checkbox',
		name = L.HidePlayerFrame,
		getFunc = function()
			return cfg.hideDefaultPlayerFrame
		end,
		setFunc = function(arg)
			if(arg ~=  cfg.hideDefaultPlayerFrame) then
				cfg.hideDefaultPlayerFrame = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkHidePlayer:SetAnchor(TOPLEFT)
	
	local chkHideTarget =  AddControlToSub(subMenu,{
		type = 'checkbox',
		name = L.HideTargetFrame,
		getFunc = function()
			return cfg.hideDefaultTargetFrame
		end,
		setFunc = function(arg)
			if(arg ~=  cfg.hideDefaultTargetFrame) then
				cfg.hideDefaultTargetFrame = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkHideTarget:SetAnchor(TOPLEFT,chkHidePlayer,BOTTOMLEFT,0,10)
	
	local dropLeftStyle = AddControlToSub(subMenu,{
		type = 'dropdown',
		name = L.ContainerLeftStyle, 
		tooltip = L.ContainerLeftStyleTooltip, 
		choices = {L.ContainerStyleNormal,L.ContainerStyleStairway},
		getFunc = function()
			if(cfg.barContainerLeft.stepOn) then
				return L.ContainerStyleStairway
			else
				return L.ContainerStyleNormal
			end
		end,
		setFunc = function(arg)
			if(arg == L.ContainerStyleNormal and cfg.barContainerLeft.stepOn) then
				cfg.barContainerLeft.stepOn = false
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			elseif(arg == L.ContainerStyleStairway and not cfg.barContainerLeft.stepOn) then
				cfg.barContainerLeft.stepOn = true
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	dropLeftStyle:SetAnchor(TOPLEFT,chkHideTarget,BOTTOMLEFT,0,10)
	
	local dropRightSide = AddControlToSub(subMenu,{
		type = 'dropdown',
		name = L.ContainerRightStyle, 
		tooltip = L.ContainerRightStyleTooltip, 
		choices = {L.ContainerStyleNormal,L.ContainerStyleStairway},
		getFunc = function()
			if(cfg.barContainerRight.stepOn) then
				return L.ContainerStyleStairway
			else
				return L.ContainerStyleNormal
			end
		end,
		setFunc = function(arg)
			if(arg == L.ContainerStyleNormal and cfg.barContainerRight.stepOn) then
				cfg.barContainerRight.stepOn = false
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			elseif(arg == L.ContainerStyleStairway and not cfg.barContainerRight.stepOn) then
				cfg.barContainerRight.stepOn = true
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})	
	dropRightSide:SetAnchor(TOPLEFT,dropLeftStyle,BOTTOMLEFT,0,10)
	
	local sliderBarSpacing = AddControlToSub(subMenu,{
		type = 'slider',
		name = L.BarSpacing, 
		tooltip = L.BarSpacingTooltip,
		min = -5,
		max = 50,
		step =1,
		getFunc = function() return cfg.barSpacing end,
		setFunc = function(arg)
			if(arg ~=  cfg.barSpacing) then
				cfg.barSpacing = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	sliderBarSpacing:SetAnchor(TOPLEFT,dropRightSide,BOTTOMLEFT,0,10)
	
	local sliderBarWidth = AddControlToSub(subMenu,{
		type = 'slider',
		name = L.BarWidth, 
		tooltip = L.BarWidthTooltip, 
		min = 20,
		max = 100, 
		step = 1,
		getFunc = function() return cfg.barWidth end,
		setFunc = function(arg)
			if(arg ~=  cfg.barWidth) then
				cfg.barWidth = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	sliderBarWidth:SetAnchor(TOPLEFT,sliderBarSpacing,BOTTOMLEFT,0,10)
	
	local sliderBarHeight = AddControlToSub(subMenu,{
		type = 'slider',
		name = L.BarHeight,
		tooltip = L.BarHeightTooltip, 
		min = 140,
		max = 700, 
		step = 1,
		getFunc = function() return cfg.barHeight end,
		setFunc = function(arg)
			if(arg ~=  cfg.barHeight) then
				cfg.barHeight = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	sliderBarHeight:SetAnchor(TOPLEFT,sliderBarWidth,BOTTOMLEFT,0,10)

	local oocSlider = AddControlToSub(subMenu,{
		type = 'slider',
		name = L.OocOpacity, 
		tooltip = L.OocOpacityTooltip, 
		min = 0,
		max = 100,
		step = 1,
		getFunc = function() return cfg.oocAlpha*100 end,
		setFunc = function(arg)
			if(arg/100 ~=  cfg.oocAlpha) then
				cfg.oocAlpha = arg/100
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	oocSlider:SetAnchor(TOPLEFT,sliderBarHeight,BOTTOMLEFT,0,10)
	
	local combatSlider = AddControlToSub(subMenu,{
		type = 'slider',
		name = L.CombatOpacity,
		tooltip = L.CombatOpacityTooltip,
		min = 0,
		max = 100,
		step = 1,
		getFunc = function() return cfg.combatAlpha*100 end,
		setFunc = function(arg)
			if(arg/100 ~=  cfg.combatAlpha) then
				cfg.combatAlpha = arg/100
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	combatSlider:SetAnchor(TOPLEFT,oocSlider,BOTTOMLEFT,0,10)
		
	local targetAcquiredSlider = AddControlToSub(subMenu,{
		type = 'slider',
		name = L.TargetAcquiredOpacity,
		tooltip = L.TargetAcquiredOpacityTooltip,
		min = 0,
		max = 100, 
		step = 1,
		getFunc = function() return cfg.targetAcquiredAlpha*100 end,
		setFunc = function(arg)
			if(arg/100 ~=  cfg.targetAcquiredAlpha) then
				cfg.targetAcquiredAlpha = arg/100
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	targetAcquiredSlider:SetAnchor(TOPLEFT,combatSlider,BOTTOMLEFT,0,10)
	
	local attributeUsedSlider = AddControlToSub(subMenu,{
		type = 'slider',
		name = L.AttributeUsedOpacity,
		tooltip = L.AttributeUsedOpacityTooltip,
		min = 0,
		max = 100,
		step = 1,
		getFunc = function() return cfg.attributeUsedAlpha*100 end,
		setFunc = function(arg)
			if(arg/100 ~=  cfg.attributeUsedAlpha) then
				cfg.attributeUsedAlpha = arg/100
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	attributeUsedSlider:SetAnchor(TOPLEFT,targetAcquiredSlider,BOTTOMLEFT,0,10)
	
	lastAddedControl = subMenu
end

function CurvyHud:BuildTargetNameplateSettings()
	local nameplateCfg = self.config.targetNameplate
	
	local subMenu = AddControl({
		type = 'submenu',
		name = headerColor..L.TargetNameplateSettings,
	})
	subMenu:SetAnchor(TOPLEFT,lastAddedControl,BOTTOMLEFT,0,10)
	
	local chkShowNameplate = AddControlToSub(subMenu, {
		type = 'checkbox',
		name = L.ShowNameplate, 
		getFunc = function()
			return nameplateCfg.show
		end,
		setFunc = function(arg)
			if(arg ~= nameplateCfg.show) then
				nameplateCfg.show = arg
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkShowNameplate:SetAnchor(TOPLEFT)
	
	local dropFont = AddControlToSub(subMenu, {  
		type = 'dropdown',
		name = L.Font,
		choices = GetLMPFonts(),
		getFunc = function()
			return nameplateCfg.fontName		
		end,
		setFunc = function(arg)
			if(nameplateCfg.fontName ~= arg) then
				nameplateCfg.fontName = arg	
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	dropFont:SetAnchor(TOPLEFT,chkShowNameplate,BOTTOMLEFT,0,10)
	
	
	local sliderFontSize = AddControlToSub(subMenu, {  
		type = 'slider',
		name = L.FontSize, 
		min = 1,
		max = 50,
		step =1,
		getFunc = function() return nameplateCfg.fontSize end,
		setFunc = function(arg)
			if(arg ~=  nameplateCfg.fontSize) then
				nameplateCfg.fontSize = arg
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	sliderFontSize:SetAnchor(TOPLEFT,dropFont,BOTTOMLEFT,0,10)
	
	local chkClassIcon = AddControlToSub(subMenu, {  
		type = 'checkbox',
		name = L.ClassIconDisplay,
		getFunc = function()
			return nameplateCfg.classIcon
		end,
		setFunc = function(arg)
			if(arg ~= nameplateCfg.classIcon) then
				nameplateCfg.classIcon = arg
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkClassIcon:SetAnchor(TOPLEFT,sliderFontSize,BOTTOMLEFT,0,10)
	
	local chkRankIcon = AddControlToSub(subMenu, {  
		type = 'checkbox',
		name = L.RankIconDisplay,
		getFunc = function()
			return nameplateCfg.rankIcon
		end,
		setFunc = function(arg)
			if(arg ~= nameplateCfg.rankIcon) then
				nameplateCfg.rankIcon = arg
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkRankIcon:SetAnchor(TOPLEFT,chkClassIcon,BOTTOMLEFT,0,10)
	
	local chkAllianceIcon = AddControlToSub(subMenu, {
		type = 'checkbox',
		name = L.AllianceIconDisplay,
		getFunc = function()
			return nameplateCfg.allianceIcon
		end,
		setFunc = function(arg)
			if(arg ~= nameplateCfg.allianceIcon) then
				nameplateCfg.allianceIcon = arg
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkAllianceIcon:SetAnchor(TOPLEFT,chkRankIcon,BOTTOMLEFT,0,10)
	
	local dropReactionColor = AddControlToSub(subMenu, {  
		type = 'dropdown',
		name = L.ColorLevelBy, 
		choices = {L.None, L.Reaction,L.Difficulty},
		getFunc = function()
			if(nameplateCfg.colorLevel == 'reaction') then
				return L.Reaction
			elseif(nameplateCfg.colorLevel == 'difficulty') then
				return L.Difficulty
			else
				return L.None
			end
		end,
		setFunc = function(arg)
			if(arg == L.Reaction and (nameplateCfg.colorLevel == 'difficulty' or nameplateCfg.colorLevel == nil)) then
				nameplateCfg.colorLevel = 'reaction'
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			elseif(arg == L.Difficulty and (nameplateCfg.colorLevel == 'reaction' or nameplateCfg.colorLevel == nil)) then
				nameplateCfg.colorLevel = 'difficulty'
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			elseif(arg == L.None and (nameplateCfg.colorLevel == 'reaction' or nameplateCfg.colorLevel == 'difficulty')) then
				nameplateCfg.colorLevel = nil
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	dropReactionColor:SetAnchor(TOPLEFT,chkAllianceIcon,BOTTOMLEFT,0,10)
	
	local dropColorBy = AddControlToSub(subMenu, {  
		type = 'dropdown',
		name = 	L.ColorNameBy, 
		choices = {L.None, L.Reaction,L.Difficulty},
		getFunc = function()
			if(nameplateCfg.colorName == 'reaction') then
				return L.Reaction
			elseif(nameplateCfg.colorName == 'difficulty') then
				return L.Difficulty
			else
				return L.None
			end
		end,
		setFunc = function(arg)
			if(arg == L.Reaction and (nameplateCfg.colorName == 'difficulty' or nameplateCfg.colorName == nil)) then
				nameplateCfg.colorName = 'reaction'
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			elseif(arg == L.Difficulty and (nameplateCfg.colorName == 'reaction' or nameplateCfg.colorName == nil)) then
				nameplateCfg.colorName = 'difficulty'
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			elseif(arg == L.None and (nameplateCfg.colorName == 'reaction'  or nameplateCfg.colorName == 'difficulty')) then
				nameplateCfg.colorName = nil
				self:BuildTargetNameplate()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	dropColorBy:SetAnchor(TOPLEFT,dropReactionColor,BOTTOMLEFT,0,10)
	
	--
	lastAddedControl = subMenu
end

function CurvyHud:BuildBarSettings(barName,barSubMenu,topAnchor)
	local cfg = self.config[barName]
	local panel
	panel = WINDOW_MANAGER:CreateControl(nil, barSubMenu.scroll, CT_CONTROL)
	panel:SetWidth(510)
	panel:SetResizeToFitDescendents(true)
	panel:SetAnchor(TOPLEFT,topAnchor,BOTTOMLEFT)
	
	local result = {}
	
	local chkShow = AddControlToSub(barSubMenu,{
		type = 'checkbox',
		name = L.ShowBar,
		getFunc = function() return cfg.show	end,
		setFunc = function(arg)
			if(arg ~= cfg.show) then
				cfg.show = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})	
	chkShow:SetAnchor(TOPLEFT,panel,TOPLEFT,0,10)
	table.insert(result, chkShow)
	
	local dropPosition = AddControlToSub(barSubMenu,{
		type = 'dropdown',
		name = L.Position, 
		choices = {L.Left, L.Right},
		getFunc = function()
			if(cfg.position	== 'left' )then
				return L.Left
			elseif (cfg.position == 'right') then
				return L.Right
			end
		end,
		setFunc = function(arg)
			local tmp
			if(arg	== L.Left) then
				tmp = 'left'
			elseif (arg == L.Right) then
				tmp =  'right'
			end
			
			if(cfg.position ~= tmp) then
				cfg.position = tmp	
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	dropPosition:SetAnchor(TOPLEFT,chkShow,BOTTOMLEFT,0,10)
	table.insert(result, dropPosition)
	
	local dropIndex = AddControlToSub(barSubMenu,{
		type = 'dropdown',
		name = L.Index, 
		tooltip = L.IndexTooltip, 
		choices = { 1, 2, 3,4,5,6},
		getFunc = function()
			return cfg.index +1	
		end,
		setFunc = function(arg)
			if(cfg.index ~= arg -1) then
				cfg.index = arg	-1
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	dropIndex:SetAnchor(TOPLEFT,dropPosition,BOTTOMLEFT,0,10)
	table.insert(result, dropIndex)
	
	local colorMax = AddControlToSub(barSubMenu,{
		type =	'colorpicker',
		name = L.BarColorMax, 
		tooltip = L.BarColorMaxTooltip,
		getFunc = function()
			return unpack(cfg.colorMax)
		end,
		setFunc = function(r,g,b,a)
			cfg.colorMax = {r,g,b,a}
			self:InitializeControls()
			self:InitializeState()
			somethingChanged = true
		end
	})
	colorMax:SetAnchor(TOPLEFT,dropIndex,BOTTOMLEFT,0,10)
	table.insert(result,colorMax)
	
	local colorLow = AddControlToSub(barSubMenu,{
		type = 'colorpicker',
		name = L.BarColorLow,
		tooltip = L.BarColorLowTooltip,
		getFunc =function()
			return unpack(cfg.colorLow)
		end,
		setFunc = function(r,g,b,a)
			cfg.colorLow = {r,g,b,a}
			self:InitializeControls()
			self:InitializeState()
			somethingChanged = true
		end
	})
	colorLow:SetAnchor(TOPLEFT,colorMax,BOTTOMLEFT,0,10)
	table.insert(result,colorLow)
	
	local borderColorMax = AddControlToSub(barSubMenu,{
		type = 'colorpicker',
		name = L.BorderColorMax,
		tooltip = L.BorderColorMaxTooltip,
		getFunc = function()
			return unpack(cfg.borderColorMax)
		end,
		setFunc = function(r,g,b,a)
			cfg.borderColorMax = {r,g,b,a}
			self:InitializeControls()
			self:InitializeState()
			somethingChanged = true
		end
	})
	borderColorMax:SetAnchor(TOPLEFT,colorLow,BOTTOMLEFT,0,10)
	table.insert(result,borderColorMax)
	
	local borderColorLow = AddControlToSub(barSubMenu,{
		type = 'colorpicker',
		name = L.BorderColorLow,
		tooltip = L.BorderColorLowTooltip ,
		getFunc =function()
			return unpack(cfg.borderColorLow)
		end,
		setFunc = function(r,g,b,a)
			cfg.borderColorLow = {r,g,b,a}
			self:InitializeControls()
			self:InitializeState()
			somethingChanged = true
		end
	})
	borderColorLow:SetAnchor(TOPLEFT,borderColorMax,BOTTOMLEFT,0,10)
	table.insert(result,borderColorLow)
	
	
	local chkShowText = AddControlToSub(barSubMenu,{
		type = 'checkbox',
		name = L.ShowText,
		getFunc = function()
			return cfg.showText
		end,
		setFunc = function(arg)
			if(arg ~= cfg.showText) then
				cfg.showText = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkShowText:SetAnchor(TOPLEFT,borderColorLow,BOTTOMLEFT,0,10)
	table.insert(result,chkShowText)
	
	local dropFont = AddControlToSub(barSubMenu,{
		type = 'dropdown', 
		name = L.Font,
		choices = GetLMPFonts(),
		getFunc = function()
			return cfg.fontName		
		end,
		setFunc = function(arg)
			if(cfg.fontName ~= arg) then
				cfg.fontName = arg	
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	dropFont:SetAnchor(TOPLEFT,chkShowText,BOTTOMLEFT,0,10)
	table.insert(result,dropFont)
	
	local sliderFontSize = AddControlToSub(barSubMenu,{
		type  = 'slider',
		name = L.FontSize, 
		min = 1,
		max =50,
		step =	1,
		getFunc = function() return cfg.fontSize end,
		setFunc = function(arg)
			if(arg ~=  cfg.fontSize) then
				cfg.fontSize = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	sliderFontSize:SetAnchor(TOPLEFT,dropFont,BOTTOMLEFT,0,10)
	table.insert(result,sliderFontSize)
	
	local editBoxFormat = AddControlToSub(barSubMenu,{
		type = 'editbox',
		name = L.TextFormat,
		tooltip = L.TextFormatTooltip,
		isMultiline = false,
		getFunc = function()
			return cfg.textFormat
		end,
		setFunc =function(arg)
			if(arg ~= cfg.textFormat) then
				cfg.textFormat = string.gsub(string.gsub(arg,"%]",""),"%[","")
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end	
	})
	editBoxFormat:SetAnchor(TOPLEFT,sliderFontSize,BOTTOMLEFT,0,10)
	table.insert(result,editBoxFormat)

	local colorText = AddControlToSub(barSubMenu,{
		type = 'colorpicker',
		name = L.TextColor,
		getFunc = function()
			return unpack(cfg.textColor)
		end,
		setFunc = function(r,g,b,a)
			cfg.textColor = {r,g,b,a}
			self:InitializeControls()
			self:InitializeState()
			somethingChanged = true
		end
	})
	colorText:SetAnchor(TOPLEFT,editBoxFormat,BOTTOMLEFT,0,10)
	table.insert(result,colorText)
	
	for k,v in pairs(result) do
		v:SetParent(panel)
	end
	
	return panel
end

function CurvyHud:InitializeBarSettings()
	-- useful locals. Health bar for initialization
	local panels = {}
	local selectedBar = L.Health
	local divider
	
	local ShowPanel = function(barName)
		for bar,panel in pairs(panels) do
			if(barName == bar) then
				panel:SetHidden(false)
			else
				panel:SetHidden(true)
			end
		end
	end
	
	-- Setting up the header
	local subMenu = AddControl({
		type = 'submenu',
		name = headerColor..L.BarSettingsHeader
	})
	subMenu:SetAnchor(TOPLEFT,lastAddedControl,BOTTOMLEFT,0,10)
	
	
	local dropBars = AddControlToSub(subMenu,{
		type = 'dropdown',
		name = L.BarSelect, 
		choices = {L.Health, L.Stamina,L.Magicka,L.Target,L.Mount,L.Siege,L.Werewolf},
		getFunc = function()			
			return selectedBar		
		end,
		setFunc = function(arg)
			if(arg ~= nil and arg ~= '' and selectedBar ~= arg) then
				selectedBar = arg
				local barName = dropBarList[arg]
				
				
				if(panels[barName] == nil) then
					panels[barName] = self:BuildBarSettings(barName,subMenu,divider)
				end

				ShowPanel(barName)
			end
		end
	})
	dropBars:SetAnchor(TOPLEFT)
	
	divider = WINDOW_MANAGER:CreateControlFromVirtual(nil, subMenu.scroll, "ZO_Options_Divider")
	divider:SetWidth(mainPanel:GetWidth())
	divider:SetAnchor(TOPLEFT,dropBars,BOTTOMLEFT,0,10)
	divider:SetAlpha(0.5)
	

	
	panels["healthBar"] = self:BuildBarSettings("healthBar",subMenu,divider)
	lastAddedControl = subMenu
end

function CurvyHud:InitializeVisualsSettings()
	local cfg = self.config.visuals
	
	local subMenu = AddControl({
		type = 'submenu',
		name = headerColor..L.VisualsSettingsHeader,
	})
	subMenu:SetAnchor(TOPLEFT,lastAddedControl,BOTTOMLEFT,0,10)
	
	local chkShowShield = AddControlToSub(subMenu, {
		type = 'checkbox',
		name = L.ShowShield, 
		getFunc = function()
			return cfg.shield.show
		end,
		setFunc = function(arg)
			if(arg ~= cfg.shield.show) then
				cfg.shield.show = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkShowShield:SetAnchor(TOPLEFT)
	
	local color = AddControlToSub(subMenu,{
		type =	'colorpicker',
		name = L.ShieldColor, 
		getFunc = function()
			return unpack(cfg.shield.color)
		end,
		setFunc = function(r,g,b,a)
			cfg.shield.color = {r,g,b,a}
			self:InitializeControls()
			self:InitializeState()
			somethingChanged = true
		end
	})
	color:SetAnchor(TOPLEFT,chkShowShield,BOTTOMLEFT,0,10)
	
	local chkShowText = AddControlToSub(subMenu,{
		type = 'checkbox',
		name = L.ShowShieldText,
		getFunc = function()
			return cfg.shield.showText
		end,
		setFunc = function(arg)
			if(arg ~= cfg.shield.showText) then
				cfg.shield.showText = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkShowText:SetAnchor(TOPLEFT,color,BOTTOMLEFT,0,10)
	
	local dropFont = AddControlToSub(subMenu,{
		type = 'dropdown', 
		name = L.Font,
		choices = GetLMPFonts(),
		getFunc = function()
			return cfg.shield.fontName		
		end,
		setFunc = function(arg)
			if(cfg.shield.fontName ~= arg) then
				cfg.shield.fontName = arg	
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	dropFont:SetAnchor(TOPLEFT,chkShowText,BOTTOMLEFT,0,10)
	
	local sliderFontSize = AddControlToSub(subMenu,{
		type  = 'slider',
		name = L.FontSize, 
		min = 1,
		max =50,
		step =	1,
		getFunc = function() return cfg.shield.fontSize end,
		setFunc = function(arg)
			if(arg ~=  cfg.shield.fontSize) then
				cfg.shield.fontSize = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	sliderFontSize:SetAnchor(TOPLEFT,dropFont,BOTTOMLEFT,0,10)
	
	local editBoxFormat = AddControlToSub(subMenu,{
		type = 'editbox',
		name = L.ShieldTextFormat,
		tooltip = L.ShieldTextFormatTooltip,
		isMultiline = false,
		getFunc = function()
			return cfg.shield.textFormat
		end,
		setFunc =function(arg)
			if(arg ~= cfg.shield.textFormat) then
				cfg.shield.textFormat = string.gsub(string.gsub(arg,"%]",""),"%[","")
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end	
	})
	editBoxFormat:SetAnchor(TOPLEFT,sliderFontSize,BOTTOMLEFT,0,10)

	local colorText = AddControlToSub(subMenu,{
		type = 'colorpicker',
		name = L.ShieldTextColor,
		getFunc = function()
			return unpack(cfg.shield.textColor)
		end,
		setFunc = function(r,g,b,a)
			cfg.shield.textColor = {r,g,b,a}
			self:InitializeControls()
			self:InitializeState()
			somethingChanged = true
		end
	})
	colorText:SetAnchor(TOPLEFT,editBoxFormat,BOTTOMLEFT,0,10)
	
	local chkShowRegen = AddControlToSub(subMenu, {
		type = 'checkbox',
		name = L.ShowRegen, 
		getFunc = function()
			return cfg.regen.show
		end,
		setFunc = function(arg)
			if(arg ~= cfg.regen.show) then
				cfg.regen.show = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkShowRegen:SetAnchor(TOPLEFT,colorText,BOTTOMLEFT,0,10)
	
	local chkShowDegen = AddControlToSub(subMenu, {
		type = 'checkbox',
		name = L.ShowDegen, 
		getFunc = function()
			return cfg.degen.show
		end,
		setFunc = function(arg)
			if(arg ~= cfg.degen.show) then
				cfg.degen.show = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkShowDegen:SetAnchor(TOPLEFT,chkShowRegen,BOTTOMLEFT,0,10)
	
	lastAddedControl = subMenu
end

function CurvyHud:InitializeTargetVisualsSettings()
	local cfg = self.config.targetVisuals
	
	local subMenu = AddControl({
		type = 'submenu',
		name = headerColor..L.TargetVisualsSettingsHeader,
	})
	subMenu:SetAnchor(TOPLEFT,lastAddedControl,BOTTOMLEFT,0,10)
	
	local chkShowShield = AddControlToSub(subMenu, {
		type = 'checkbox',
		name = L.ShowShield, 
		getFunc = function()
			return cfg.shield.show
		end,
		setFunc = function(arg)
			if(arg ~= cfg.shield.show) then
				cfg.shield.show = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkShowShield:SetAnchor(TOPLEFT)
	
	local color = AddControlToSub(subMenu,{
		type =	'colorpicker',
		name = L.ShieldColor, 
		getFunc = function()
			return unpack(cfg.shield.color)
		end,
		setFunc = function(r,g,b,a)
			cfg.shield.color = {r,g,b,a}
			self:InitializeControls()
			self:InitializeState()
			somethingChanged = true
		end
	})
	color:SetAnchor(TOPLEFT,chkShowShield,BOTTOMLEFT,0,10)
	
	local chkShowText = AddControlToSub(subMenu,{
		type = 'checkbox',
		name = L.ShowShieldText,
		getFunc = function()
			return cfg.shield.showText
		end,
		setFunc = function(arg)
			if(arg ~= cfg.shield.showText) then
				cfg.shield.showText = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkShowText:SetAnchor(TOPLEFT,color,BOTTOMLEFT,0,10)
	
	local dropFont = AddControlToSub(subMenu,{
		type = 'dropdown', 
		name = L.Font,
		choices = GetLMPFonts(),
		getFunc = function()
			return cfg.shield.fontName		
		end,
		setFunc = function(arg)
			if(cfg.shield.fontName ~= arg) then
				cfg.shield.fontName = arg	
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	dropFont:SetAnchor(TOPLEFT,chkShowText,BOTTOMLEFT,0,10)
	
	local sliderFontSize = AddControlToSub(subMenu,{
		type  = 'slider',
		name = L.FontSize, 
		min = 1,
		max =50,
		step =	1,
		getFunc = function() return cfg.shield.fontSize end,
		setFunc = function(arg)
			if(arg ~=  cfg.shield.fontSize) then
				cfg.shield.fontSize = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	sliderFontSize:SetAnchor(TOPLEFT,dropFont,BOTTOMLEFT,0,10)
	
	local editBoxFormat = AddControlToSub(subMenu,{
		type = 'editbox',
		name = L.ShieldTextFormat,
		tooltip = L.ShieldTextFormatTooltip,
		isMultiline = false,
		getFunc = function()
			return cfg.shield.textFormat
		end,
		setFunc =function(arg)
			if(arg ~= cfg.shield.textFormat) then
				cfg.shield.textFormat = string.gsub(string.gsub(arg,"%]",""),"%[","")
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end	
	})
	editBoxFormat:SetAnchor(TOPLEFT,sliderFontSize,BOTTOMLEFT,0,10)

	local colorText = AddControlToSub(subMenu,{
		type = 'colorpicker',
		name = L.ShieldTextColor,
		getFunc = function()
			return unpack(cfg.shield.textColor)
		end,
		setFunc = function(r,g,b,a)
			cfg.shield.textColor = {r,g,b,a}
			self:InitializeControls()
			self:InitializeState()
			somethingChanged = true
		end
	})
	colorText:SetAnchor(TOPLEFT,editBoxFormat,BOTTOMLEFT,0,10)
	
	local chkShowRegen = AddControlToSub(subMenu, {
		type = 'checkbox',
		name = L.ShowRegen, 
		getFunc = function()
			return cfg.regen.show
		end,
		setFunc = function(arg)
			if(arg ~= cfg.regen.show) then
				cfg.regen.show = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkShowRegen:SetAnchor(TOPLEFT,colorText,BOTTOMLEFT,0,10)
	
	local chkShowDegen = AddControlToSub(subMenu, {
		type = 'checkbox',
		name = L.ShowDegen, 
		getFunc = function()
			return cfg.degen.show
		end,
		setFunc = function(arg)
			if(arg ~= cfg.degen.show) then
				cfg.degen.show = arg
				self:InitializeControls()
				self:InitializeState()
				somethingChanged = true
			end
		end
	})
	chkShowDegen:SetAnchor(TOPLEFT,chkShowRegen,BOTTOMLEFT,0,10)
	
	lastAddedControl = subMenu
end