local function round(num) 
	if num >= 0 then return math.floor(num+.5) 
	else return math.ceil(num-.5) end
end


function CurvyHud:InitializeMoveMode()
	-- Initializing all movable frames to support movement
	for frameName,frame in pairs(self.movableFrames) do
		self:BuildMoveBackdrop(frameName,frame)
		
		-- when a frame start moving, we continuously update the related setter and configuration
		-- hopefully, frames are only movable in move mode
		frame:SetHandler("OnMoveStart", 
			function(self) 	
				EVENT_MANAGER:RegisterForUpdate('CurvyHud', 50,
					function()
						if(CurvyHud.positionPanel ~= nil) then
							CurvyHud:UpdatePosition(frameName, self)
						end
					end
				)
			end
		)
		
		frame:SetHandler("OnMoveStop",
			function(self)
				EVENT_MANAGER:UnregisterForUpdate('CurvyHud')
			end
		)
		
	end
	
	-- Adding slash commands
	SLASH_COMMANDS["/curvy_move"] = function(active)
		if(active == "on") then
				self:ToggleMoveFramesMode(true)
		elseif(active == "off") then
				self:ToggleMoveFramesMode(false)
		end
	end
	
	
	
	-- self.moveMode.lockContainers = false
	self.moveMode.lockTextValues = false
end

function CurvyHud:BuildMoveBackdrop(frameName,frame)
	local c = frame

	c.bg = WINDOW_MANAGER:CreateControl(nil,c,CT_BACKDROP)
	c.bg:SetCenterColor(1,1,1,0.2)
	c.bg:SetEdgeColor(1,1,1,0.5)
	c.bg:SetEdgeTexture('',8,1,3)
	c.bg:SetAnchorFill(c)
	c.bg:SetHidden(true)
end

function CurvyHud:BuildPositionPanel()
	local cfg = self.config
	-- Creating the positions panel
	local panel = WINDOW_MANAGER:CreateTopLevelWindow('Curvy_MoveModePanel')
	panel:SetAnchor(BOTTOMRIGHT)
	panel:SetDimensions(400,300)
	panel:SetMovable(true)
	panel:SetClampedToScreen(true)
	panel:SetClampedToScreenInsets(10,10,10,10)
	panel:SetMouseEnabled(true)
	panel.bg = WINDOW_MANAGER:CreateControlFromVirtual(nil, panel, "ZO_DefaultBackdrop")
	
	-- header
	local title = WINDOW_MANAGER:CreateControl(nil, panel,CT_LABEL)
	title:SetFont("ZoFontHeader3")
	title:SetDimensions(panel:GetWidth(),40)
	title:SetAnchor(TOP)
	title:SetVerticalAlignment(TEXT_ALIGN_TOP)
	title:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
	title:SetText("Frames coordinates")
	panel:SetHidden(false)
	panel.header = title
	
	-- creating setters for all movable frames
	-- could loop on it, but we want some kind of order
	panel.setters = {}
	local setters = panel.setters
	
	local c = title
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_barContainerLeft"],self.barContainerLeft,c)
	setters['barContainerLeft'] = c 
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_barContainerRight"],self.barContainerRight,c)
	setters['barContainerRight'] = c 
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_targetNameplate"],self.targetNameplate,c)
	setters['targetNameplate'] = c 
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_interactionPrompt"],self.interactionPrompt,c)
	setters['interactionPrompt'] = c 
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_healthBarText"],self.bars.healthBar.textContainer,c)
	setters['healthBarText'] = c 
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_magickaBarText"],self.bars.magickaBar.textContainer,c)
	setters['magickaBarText'] = c 
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_staminaBarText"],self.bars.staminaBar.textContainer,c)
	setters['staminaBarText'] = c 
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_targetBarText"],self.bars.targetBar.textContainer,c)
	setters['targetBarText'] = c
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_mountBarText"],self.bars.mountBar.textContainer,c)
	setters['mountBarText'] = c
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_werewolfBarText"],self.bars.werewolfBar.textContainer,c)
	setters['werewolfBarText'] = c
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_siegeBarText"],self.bars.siegeBar.textContainer,c)
	setters['siegeBarText'] = c
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_shieldText"],self.bars.healthBar.shield.textContainer,c)
	setters['shieldText'] = c
	c = self:CreatePositionSetter(panel,self:GetLoc()["Move_targetShieldText"],self.bars.targetBar.shield.textContainer,c)
	setters['targetShieldText'] = c 
	
	
	panel:SetHeight(math.abs(title:GetTop() - c:GetBottom()))
	self.positionPanel = panel
end

function CurvyHud:CreatePositionSetter(parent,labelText,frame,previousControl)
	local control = WINDOW_MANAGER:CreateControl(nil, parent,CT_CONTROL)
	local anchorConfig = frame.anchorConfig
	control:SetAnchor(TOPLEFT,previousControl,BOTTOMLEFT)
	control:SetDimensions(parent:GetWidth(),26)
	
	-- label
	local label = WINDOW_MANAGER:CreateControl(nil,control,CT_LABEL)
	label:SetDimensions(170, 26)
	label:SetAnchor(TOPLEFT)
	label:SetFont("ZoFontWinH4")
	label:SetText(labelText)
	
	-- X label
	local Xlabel = WINDOW_MANAGER:CreateControl(nil,control,CT_LABEL)
	Xlabel:SetDimensions(30, 26)
	Xlabel:SetAnchor(LEFT,label,RIGHT,0,0)
	Xlabel:SetFont("ZoFontWinH4")
	Xlabel:SetText("X = ")
	
	local Xwarning = {}
	Xwarning.tooltipText = ''
	local XeditBoxBg = WINDOW_MANAGER:CreateControlFromVirtual(nil, control, "ZO_EditBackdrop")
	XeditBoxBg:SetDimensions(50,24)
	XeditBoxBg:SetAnchor(LEFT,Xlabel,RIGHT,0,-3)
	XeditBox = WINDOW_MANAGER:CreateControlFromVirtual(nil, XeditBoxBg, "ZO_DefaultEditForBackdrop")
	XeditBox:SetText(anchorConfig[4])
	XeditBox:SetHandler("OnFocusLost", 
		function(self)
			local text = self:GetText()
			local _, x = string.gsub(text,"[^0-9-]","")
			if(x > 0 or tonumber(text) == nil) then
				Xwarning.tooltipText = CurvyHud:GetLoc()["PositionSetterWarningNaN"]
				Xwarning:SetAlpha(1)
			else
				Xwarning.tooltipText = ""
				Xwarning:SetAlpha(0)
				frame.anchorConfig[4] = text
				frame:ClearAnchors()
				frame:SetAnchor(unpack(frame.anchorConfig))
			end
		end
	)
	XeditBox:SetHandler("OnEscape", function(self) self:LoseFocus() end)
	XeditBox:SetMaxInputChars(5)
	
	Xwarning = WINDOW_MANAGER:CreateControlFromVirtual(nil, control, "ZO_Options_WarningIcon")
	Xwarning:SetAnchor(TOPLEFT, XeditBox, TOPRIGHT, 5, -3)
	Xwarning.tooltipText = ''
	Xwarning:SetAlpha(0)
	
	-- Y label
	local Ylabel = WINDOW_MANAGER:CreateControl(nil,control,CT_LABEL)
	Ylabel:SetDimensions(35, 26)
	Ylabel:SetAnchor(LEFT,Xlabel,RIGHT,90,0)
	Ylabel:SetFont("ZoFontWinH4")
	Ylabel:SetText("Y = ")
	
	local Ywarning = {}
	Ywarning.tooltipText = ''
	local YeditBoxBg = WINDOW_MANAGER:CreateControlFromVirtual(nil, control, "ZO_EditBackdrop")
	YeditBoxBg:SetDimensions(50,24)
	YeditBoxBg:SetAnchor(LEFT,Ylabel,RIGHT,0,-3)
	YeditBox = WINDOW_MANAGER:CreateControlFromVirtual(nil, YeditBoxBg, "ZO_DefaultEditForBackdrop")
	YeditBox:SetText(anchorConfig[5])
	YeditBox:SetHandler("OnFocusLost", 
		function(self)
			local text = self:GetText()
			local _, x = string.gsub(text,"[^0-9-]","")
			if(x > 0 or tonumber(text) == nil) then
				Ywarning.tooltipText = CurvyHud:GetLoc()["PositionSetterWarningNaN"]
				Ywarning:SetAlpha(1)
			else
				Ywarning.tooltipText = ""
				Ywarning:SetAlpha(0)
				frame.anchorConfig[5] = text
				frame:ClearAnchors()
				frame:SetAnchor(unpack(frame.anchorConfig))
			end
		end
	)
	YeditBox:SetHandler("OnEscape", function(self) self:LoseFocus() end)
	YeditBox:SetMaxInputChars(5)
	
	Ywarning = WINDOW_MANAGER:CreateControlFromVirtual(nil, control, "ZO_Options_WarningIcon")
	Ywarning:SetAnchor(TOPLEFT, YeditBox, TOPRIGHT, 5, -3)
	Ywarning.tooltipText = ''
	Ywarning:SetAlpha(0)
	
	control.Xedit = XeditBox
	control.Yedit = YeditBox
	return control
end

function CurvyHud:ToggleMoveFramesMode(enable)
	if(enable == true) then
		-- building position panel if not built yet, and showing it
		if(self.positionPanel == nil) then
			self:BuildPositionPanel()
		end
		self.positionPanel:SetHidden(false)
		
		-- changing state
		self.moveMode.enabled = true
		CurvyHudOptions_EnableMoveBtn.button:SetState(BSTATE_DISABLED)
		CurvyHudOptions_DisableMoveBtn.button:SetState(BSTATE_NORMAL)
		
		-- enabling frame movement, showing backdrops and setting handlers
		for frameName,frame in pairs(self.movableFrames) do
			frame:SetMovable(true)
			frame:SetClampedToScreen(true)
			frame:SetMouseEnabled(true)
			frame.bg:SetHidden(false)
			

			frame:SetHandler("OnMouseEnter",
				function(self)
					for setterName,setter in pairs(CurvyHud.positionPanel.setters) do
						if(setterName == frameName) then
							setter:SetAlpha(1)
						else
							setter:SetAlpha(0.2)
						end
					end
				end
			)
			
			frame:SetHandler("OnMouseExit",
				function(self)
					for _,setter in pairs(CurvyHud.positionPanel.setters) do
						setter:SetAlpha(1)
					end
				end
			)
		end
		
		-- unregistering for events which hide the HUD (we want to always show it as we move things)
		EVENT_MANAGER:UnregisterForEvent('CurvyHud', EVENT_RETICLE_HIDDEN_UPDATE)
		EVENT_MANAGER:UnregisterForEvent('CurvyHud', EVENT_RETICLE_TARGET_CHANGED)
		EVENT_MANAGER:UnregisterForEvent('CurvyHud', EVENT_POWER_UPDATE)
		EVENT_MANAGER:UnregisterForEvent('CurvyHud', EVENT_PLAYER_COMBAT_STATE)
	elseif(not enable) then
	
		-- Hiding all movable frame an position panel
		if(self.positionPanel ~= nil) then
			self.positionPanel:SetHidden(true)
		end
		
		-- changing state
		self.moveMode.enabled = false
		
		CurvyHudOptions_EnableMoveBtn.button:SetState(BSTATE_NORMAL)
		CurvyHudOptions_DisableMoveBtn.button:SetState(BSTATE_DISABLED)
		
		-- enabling frame movement, hiding background and nil-ing handlers
		for frameName,frame in pairs(self.movableFrames) do
			frame:SetMovable(false)
			frame:SetClampedToScreen(false)
			frame:SetMouseEnabled(false)
			frame.bg:SetHidden(true)
			
			frame:SetHandler("OnMouseEnter", nil)
			frame:SetHandler("OnMouseExit", nil)
		end
		
		
		-- re-registering normal events
		EVENT_MANAGER:RegisterForEvent('CurvyHud', EVENT_RETICLE_TARGET_CHANGED, CurvyHud.OnTargetChanged)
		EVENT_MANAGER:RegisterForEvent('CurvyHud', EVENT_RETICLE_HIDDEN_UPDATE, CurvyHud.OnReticleHidden)
		EVENT_MANAGER:RegisterForEvent('CurvyHud', EVENT_POWER_UPDATE, CurvyHud.OnPowerUpdated)
		EVENT_MANAGER:RegisterForEvent('CurvyHud', EVENT_PLAYER_COMBAT_STATE, CurvyHud.OnCombatStateChanged)
	end
	
	-- reinitializing player/target bars
	self:InitPlayerBars()
	self:InitTargetBar()
	
	-- finally, show HUD
	self:ShowHud()
end

function CurvyHud:UpdatePosition(frameName,frame)
	local setter = self.positionPanel.setters[frameName]
	
	-- getting the position relative to the center
	local centerX, centerY = frame:GetCenter()
	local midX =  GuiRoot:GetWidth()/2
	local midY = GuiRoot:GetHeight()/2
	
	local newX = round(-(midX - centerX))
	local newY = round(-(midY - centerY))
	-- getting the old values
	local oldX = setter.Xedit:GetText()
	local oldY = setter.Yedit:GetText()
	
	-- updating setter in position panel
	setter.Xedit:SetText(newX)
	setter.Yedit:SetText(newY)
	
	-- saving the position in config and reset the anchor just to be sure
	frame.anchorConfig[1] = CENTER
	frame.anchorConfig[2] = GuiRoot
	frame.anchorConfig[3] = CENTER
	frame.anchorConfig[4] = newX
	frame.anchorConfig[5] = newY
	
	frame:ClearAnchors()
	frame:SetAnchor(unpack(frame.anchorConfig))
	
	local dX = newX - oldX
	local dY = newY - oldY
			
	-- if(self.moveMode.lockContainers) then
		-- if(frame == self.barContainerLeft) then
			-- self.barContainerRight.anchorConfig[1] = CENTER
			-- self.barContainerRight.anchorConfig[2] = GuiRoot
			-- self.barContainerRight.anchorConfig[3] = CENTER
	
			-- -- local X = self.barContainerRight.anchorConfig[4] + dX
			-- -- self.barContainerRight.anchorConfig[4] = X
			-- local Y = self.barContainerRight.anchorConfig[5] + dY
			-- self.barContainerRight.anchorConfig[5] = Y
			-- self.positionPanel.setters['barContainerRight'].Yedit:SetText(Y)
			
			-- self.barContainerRight:ClearAnchors()
			-- self.barContainerRight:SetAnchor(unpack(self.barContainerRight.anchorConfig))
		-- elseif(frame == self.barContainerRight) then
			-- self.barContainerLeft.anchorConfig[1] = CENTER
			-- self.barContainerLeft.anchorConfig[2] = GuiRoot
			-- self.barContainerLeft.anchorConfig[3] = CENTER
	
			-- -- local X = self.barContainerLeft.anchorConfig[4] + dX
			-- -- self.barContainerLeft.anchorConfig[4] = X
			-- local Y = self.barContainerLeft.anchorConfig[5] + dY
			-- self.barContainerLeft.anchorConfig[5] = Y
			-- self.positionPanel.setters['barContainerLeft'].Yedit:SetText(Y)
			
			-- self.barContainerLeft:ClearAnchors()
			-- self.barContainerLeft:SetAnchor(unpack(self.barContainerLeft.anchorConfig))
		-- end
	-- end
	
	if(self.moveMode.lockTextValues) then
		local barContainer = nil
		if(frame == self.barContainerLeft) then
			barContainer = self.barContainerLeft
		elseif(frame == self.barContainerRight) then
			barContainer = self.barContainerRight
		end

		if(barContainer ~= nil) then
			for barName, bar in pairs(barContainer.bars) do
				local text = bar.textContainer
				local X,Y = text.anchorConfig[4]+dX, text.anchorConfig[5]+dY
				
				text.anchorConfig[1] = CENTER
				text.anchorConfig[2] = GuiRoot
				text.anchorConfig[3] = CENTER
				text.anchorConfig[4] = X
				text.anchorConfig[5] = Y
				
				text:ClearAnchors()
				text:SetAnchor(unpack(text.anchorConfig))
				self.positionPanel.setters[barName..'Text'].Xedit:SetText(X)
				self.positionPanel.setters[barName..'Text'].Yedit:SetText(Y)
			end
		end
	end
end