----------------------------------------------
-- /Visibility.lua   						--
-- Contains functions which set or alter	--
-- the visibility of the HUD controls.		--
----------------------------------------------

-- Hides or show the default player frame
function CurvyHud:ToggleDefaultPlayerFrame()
	ZO_PlayerAttributeHealth:SetHidden(self.config.hideDefaultPlayerFrame)
	ZO_PlayerAttributeMagicka:SetHidden(self.config.hideDefaultPlayerFrame)
	ZO_PlayerAttributeStamina:SetHidden(self.config.hideDefaultPlayerFrame)
	ZO_PlayerAttributeMountStamina:SetHidden(self.config.hideDefaultPlayerFrame)
	ZO_PlayerAttributeWerewolf:SetHidden(self.config.hideDefaultPlayerFrame)
	ZO_PlayerAttributeSiegeHealth:SetHidden(self.config.hideDefaultPlayerFrame)
end

-- Hides or show the default target frame
function CurvyHud:ToggleDefaultTargetFrame()
	ZO_TargetUnitFramereticleover:SetHidden(self.config.hideDefaultTargetFrame)
end	

-- Fades the target bar to the appropriate opacity
function CurvyHud:FadeTargetBar()
	local cfg = CurvyHud.config
	local targetBar = self.bars.targetBar
	local alpha = 1
	
	if(not self.state.inCombat) then
		local reaction = GetUnitReaction('reticleover')
		if(reaction == UNIT_REACTION_NEUTRAL or reaction == UNIT_REACTION_HOSTILE) then
			alpha = cfg.targetAcquiredAlpha
		else
			alpha = cfg.oocAlpha
		end	
	else
		alpha = cfg.combatAlpha
		
	end
	
	targetBar:SetAlpha(alpha)
	targetBar.textContainer:SetAlpha(alpha)
	targetBar.regen:SetGradientColors(ORIENTATION_VERTICAL,
		1,1,1,0,
		1,1,1,alpha
	)
	targetBar.degen:SetGradientColors(ORIENTATION_VERTICAL,
		1,1,1,0,
		1,1,1,alpha
	)
end

-- Show HUD and fades all controls to their appropriate opacity 
function CurvyHud:ShowHud()
	-- useful locals
	local cfg = self.config
	
	
	-- if in move mode, show all controls regardless of combat state, target, etc.
	if(self.moveMode.enabled) then
		for barName,bar in pairs(self.bars) do
			bar:SetHidden(not bar.config.show)
			bar.textContainer:SetHidden(not bar.config.showText)
			bar:SetAlpha(1)
			bar.textContainer:SetAlpha(1)
		end
		self.targetNameplate:SetHidden(false)
		self.targetNameplate:SetAlpha(1)
		self:HideHud(false)
	else
	-- 'normal' mode, we just fade all the player bars to appropriate opacity,
		for barName,bar in pairs(self.bars) do
			if(barName ~= "targetBar") then -- target bar is taken care of elsewhere
				self:FadePlayerBar(barName)
			else
				self:FadeTargetBar()
			end
		end
		
	end	
end

-- Fades a player bar to the appropriate opacity
function CurvyHud:FadePlayerBar(barName,value,maxValue)
	local bar  = self.bars[barName]
	local cfg = self.config
	local inCombat = self.state.inCombat
	
	local alpha = 0
	if(inCombat) then
		alpha = cfg.combatAlpha
	else
		if(value == nil or maxValue == nil) then
			local attribute =  {GetUnitPower('player', self.barPowertypeMapping[barName])}
			value = attribute[1]
			maxValue = attribute[3]
		end		
		if(value < maxValue or (barName == 'healthBar' and self.visuals.shield.on)) then
			alpha = cfg.attributeUsedAlpha
		else
			alpha = cfg.oocAlpha
		end
	end
	
	
	bar:SetAlpha(alpha)
	bar.regen:SetGradientColors(ORIENTATION_VERTICAL,
		1,1,1,0,
		1,1,1,alpha
	)
	bar.degen:SetGradientColors(ORIENTATION_VERTICAL,
		1,1,1,0,
		1,1,1,alpha
	)
	bar.textContainer:SetAlpha(alpha)
end

-- Toggle HUD visibility
function CurvyHud:HideHud(hide)
	if(hide) then
		self.topControl:SetHidden(true)
	else
		
		self.topControl:SetHidden(false)
	end
end

-- Manage HUD visibility with death recap
function CurvyHud:HideHudForDeathRecap(hide)
	if(hide) then
		-- basically, continuously update the HUD to check if death recap is shown or not
		EVENT_MANAGER:RegisterForUpdate('deathRecap', 100, 
			function()
				CurvyHud.OnReticleHidden(0,(not ZO_DeathRecap:IsHidden()) or IsReticleHidden())
			end
		)	
	else
		-- unregister for the death recap visibility update
		EVENT_MANAGER:UnregisterForUpdate('deathRecap')
	end
end